@@ -7,7 +7,7 @@
  * Author: Steven Dake (sdake@redhat.com)
  *
  * This software licensed under BSD license, the text of which follows:
- * 
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
  *
@@ -33,55 +33,45 @@
  * THE POSSIBILITY OF SUCH DAMAGE.
  */
 
+#include <config.h>
+
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <errno.h>
-#include <signal.h>
 #include <pthread.h>
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/select.h>
 #include <sys/un.h>
 
+#include <corosync/coroipcc.h>
+#include <corosync/coroipc_types.h>
+#include <corosync/corodefs.h>
+#include <corosync/hdb.h>
+
 #include <saAis.h>
 #include <saAmf.h>
-#include <corosync/ipc_gen.h>
 #include <ipc_amf.h>
-#include <corosync/coroipcc.h>
+
 #include "util.h"
 
-
-struct res_overlay {
-	mar_res_header_t header __attribute__((aligned(8)));
-	char data[4096];
-};
-
 /*
  * Data structure for instance data
  */
 struct amfInstance {
-	void *ipc_ctx;
+	hdb_handle_t handle;
 	SaAmfCallbacksT callbacks;
 	SaNameT compName;
 	int compRegistered;
 	int finalize;
-	pthread_mutex_t response_mutex;
-	pthread_mutex_t dispatch_mutex;
 };
 
-static void amfHandleInstanceDestructor (void *);
-
 /*
  * All instances in one database
  */
-static struct saHandleDatabase amfHandleDatabase = {
-	.handleCount				= 0,
-	.handles					= 0,
-	.mutex						= PTHREAD_MUTEX_INITIALIZER,
-	.handleInstanceDestructor	= amfHandleInstanceDestructor
-};
+DECLARE_HDB_DATABASE(amfHandleDatabase,NULL);
 
 /*
  * Versions supported
@@ -94,19 +84,11 @@ static struct saVersionDatabase amfVersionDatabase = {
 	sizeof (amfVersionsSupported) / sizeof (SaVersionT),
 	amfVersionsSupported
 };
-	
+
 /*
  * Implementation
  */
 
-void amfHandleInstanceDestructor (void *instance)
-{
-	struct amfInstance *amfInstance = instance;
-
-	pthread_mutex_destroy (&amfInstance->response_mutex);
-	pthread_mutex_destroy (&amfInstance->dispatch_mutex);
-}
-
 SaAisErrorT
 saAmfInitialize (
 	SaAmfHandleT *amfHandle,
@@ -120,36 +102,38 @@ saAmfInitialize (
 	if (error != SA_AIS_OK) {
 		goto error_no_destroy;
 	}
-	
-	error = saHandleCreate (&amfHandleDatabase, sizeof (struct amfInstance), amfHandle);
+
+	error = hdb_error_to_sa(hdb_handle_create (&amfHandleDatabase, sizeof (struct amfInstance), amfHandle));
 	if (error != SA_AIS_OK) {
 		goto error_no_destroy;
 	}
 
-	error = saHandleInstanceGet (&amfHandleDatabase, *amfHandle, (void *)&amfInstance);
+	error = hdb_error_to_sa(hdb_handle_get (&amfHandleDatabase, *amfHandle, (void *)&amfInstance));
 	if (error != SA_AIS_OK) {
 		goto error_destroy;
 	}
 
-	error = coroipcc_service_connect (IPC_SOCKET_NAME, CPG_SERVICE, &amfInstance->ipc_ctx);
+	error = coroipcc_service_connect (
+		COROSYNC_SOCKET_NAME,
+		CPG_SERVICE,
+		IPC_REQUEST_SIZE,
+		IPC_RESPONSE_SIZE,
+		IPC_DISPATCH_SIZE,
+		&amfInstance->handle);
 	if (error != SA_AIS_OK) {
 		goto error_put_destroy;
 	}
 
 	memcpy (&amfInstance->callbacks, amfCallbacks, sizeof (SaAmfCallbacksT));
 
-	pthread_mutex_init (&amfInstance->response_mutex, NULL);
-
-	pthread_mutex_init (&amfInstance->dispatch_mutex, NULL);
-
-	saHandleInstancePut (&amfHandleDatabase, *amfHandle);
+	hdb_handle_put (&amfHandleDatabase, *amfHandle);
 
 	return (SA_AIS_OK);
 
 error_put_destroy:
-	saHandleInstancePut (&amfHandleDatabase, *amfHandle);
+	hdb_handle_put (&amfHandleDatabase, *amfHandle);
 error_destroy:
-	saHandleDestroy (&amfHandleDatabase, *amfHandle);
+	hdb_handle_destroy (&amfHandleDatabase, *amfHandle);
 error_no_destroy:
 	return (error);
 }
@@ -161,16 +145,18 @@ saAmfSelectionObjectGet (
 {
 	struct amfInstance *amfInstance;
 	SaAisErrorT error;
+	int fd;
 
-	error = saHandleInstanceGet (&amfHandleDatabase, amfHandle, (void *)&amfInstance);
+	error = hdb_error_to_sa(hdb_handle_get (&amfHandleDatabase, amfHandle, (void *)&amfInstance));
 	if (error != SA_AIS_OK) {
 		return (error);
 	}
 
-	*selectionObject = coroipcc_fd_get (amfInstance->ipc_ctx);
+	error = coroipcc_fd_get (amfInstance->handle, &fd);
+	*selectionObject = fd;
 
-	saHandleInstancePut (&amfHandleDatabase, amfHandle);
-	return (SA_AIS_OK);
+	hdb_handle_put (&amfHandleDatabase, amfHandle);
+	return (error);
 }
 
 
@@ -182,7 +168,6 @@ saAmfDispatch (
 	int timeout = -1;
 	SaAisErrorT error;
 	int cont = 1; /* always continue do loop except when set to 0 */
-	int dispatch_avail;
 	struct amfInstance *amfInstance;
 	struct res_lib_amf_csisetcallback *res_lib_amf_csisetcallback;
 
@@ -190,7 +175,7 @@ saAmfDispatch (
 	struct res_lib_amf_csiremovecallback *res_lib_amf_csiremovecallback;
 	struct res_lib_amf_componentterminatecallback *res_lib_amf_componentterminatecallback;
 	SaAmfCallbacksT callbacks;
-	struct res_overlay dispatch_data;
+	coroipc_response_header_t *dispatch_data;
 
 	if (dispatchFlags != SA_DISPATCH_ONE &&
 		dispatchFlags != SA_DISPATCH_ALL &&
@@ -199,40 +184,40 @@ saAmfDispatch (
 		return (SA_AIS_ERR_INVALID_PARAM);
 	}
 
-	error = saHandleInstanceGet (&amfHandleDatabase, amfHandle,
-		(void *)&amfInstance);
+	error = hdb_error_to_sa(hdb_handle_get (&amfHandleDatabase, amfHandle,
+		(void *)&amfInstance));
 	if (error != SA_AIS_OK) {
-		return (error);
+		goto error_exit;
 	}
 
 	/*
-	 * Timeout instantly for SA_DISPATCH_ALL
+	 * Timeout instantly for SA_DISPATCH_ALL, otherwise don't timeout
+	 * for SA_DISPATCH_BLOCKING or SA_DISPATCH_ONE
 	 */
 	if (dispatchFlags == SA_DISPATCH_ALL) {
 		timeout = 0;
 	}
 
 	do {
-		dispatch_avail = coroipcc_dispatch_recv (amfInstance->ipc_ctx,
-			(void *)&dispatch_data, timeout);
-
-		pthread_mutex_lock (&amfInstance->dispatch_mutex);
-
-		/*
-		 * Handle has been finalized in another thread
-		 */
-		if (amfInstance->finalize == 1) {
-			error = SA_AIS_OK;
-			goto error_unlock;
+		error = coroipcc_dispatch_get (
+			amfInstance->handle,
+			(void **)&dispatch_data,
+			timeout);
+		if (error == CS_ERR_BAD_HANDLE) {
+			error = CS_OK;
+			goto error_put;
+		}
+		if (error != CS_OK) {
+			goto error_put;
 		}
 
-		if (dispatch_avail == 0 && dispatchFlags == SA_DISPATCH_ALL) {
-			pthread_mutex_unlock (&amfInstance->dispatch_mutex);
-			break; /* exit do while cont is 1 loop */
-		} else
-		if (dispatch_avail == 0) {
-			pthread_mutex_unlock (&amfInstance->dispatch_mutex);
-			continue; /* next poll */
+
+		if (dispatch_data == NULL) {
+			if (dispatchFlags == CPG_DISPATCH_ALL) {
+				break; /* exit do while cont is 1 loop */
+			} else {
+				continue; /* next poll */
+			}
 		}
 
 		/*
@@ -242,16 +227,15 @@ saAmfDispatch (
 		 */
 
 		memcpy (&callbacks, &amfInstance->callbacks, sizeof (SaAmfCallbacksT));
-		pthread_mutex_unlock (&amfInstance->dispatch_mutex);
 
 		/*
 		 * Dispatch incoming response
 		 */
 
-		switch (dispatch_data.header.id) {
+		switch (dispatch_data->id) {
 
 		case MESSAGE_RES_AMF_HEALTHCHECKCALLBACK:
-			res_lib_amf_healthcheckcallback = (struct res_lib_amf_healthcheckcallback *)&dispatch_data;
+			res_lib_amf_healthcheckcallback = (struct res_lib_amf_healthcheckcallback *)dispatch_data;
 
 			callbacks.saAmfHealthcheckCallback (
 				res_lib_amf_healthcheckcallback->invocation,
@@ -266,16 +250,16 @@ saAmfDispatch (
 			char *attr_buf;
 			int i;
 
-			res_lib_amf_csisetcallback = (struct res_lib_amf_csisetcallback *)&dispatch_data;
+			res_lib_amf_csisetcallback = (struct res_lib_amf_csisetcallback *)dispatch_data;
 
 
 			csi_descriptor.csiFlags = res_lib_amf_csisetcallback->csiFlags;
-			memcpy(&csi_descriptor.csiName, &res_lib_amf_csisetcallback->csiName, 
+			memcpy(&csi_descriptor.csiName, &res_lib_amf_csisetcallback->csiName,
 			       sizeof(SaNameT));
 			csi_descriptor.csiStateDescriptor = res_lib_amf_csisetcallback->csiStateDescriptor;
 			csi_descriptor.csiAttr.number = res_lib_amf_csisetcallback->number;
 
-			csi_attribute_array = malloc (sizeof (SaAmfCSIAttributeT) * 
+			csi_attribute_array = malloc (sizeof (SaAmfCSIAttributeT) *
 									  csi_descriptor.csiAttr.number);
 
 			if (csi_attribute_array == 0) {
@@ -284,7 +268,7 @@ saAmfDispatch (
 			csi_descriptor.csiAttr.attr = csi_attribute_array;
 
 			attr_buf = res_lib_amf_csisetcallback->csi_attr_buf;
-			
+
 			for (i = 0; i < csi_descriptor.csiAttr.number; i++) {
 				csi_attribute_array[i].attrName = (SaUint8T*)attr_buf;
 
@@ -293,20 +277,18 @@ saAmfDispatch (
 
 				attr_buf += strlen(attr_buf) + 1;
 			}
-			
+
 			callbacks.saAmfCSISetCallback (
 				res_lib_amf_csisetcallback->invocation,
 				&res_lib_amf_csisetcallback->compName,
 				res_lib_amf_csisetcallback->haState,
 				&csi_descriptor);
 
-			if (csi_attribute_array != NULL) {
-				free(csi_attribute_array);
-			}
+			free(csi_attribute_array);
 			break;
 		    }
 		case MESSAGE_RES_AMF_CSIREMOVECALLBACK:
-			res_lib_amf_csiremovecallback = (struct res_lib_amf_csiremovecallback *)&dispatch_data;
+			res_lib_amf_csiremovecallback = (struct res_lib_amf_csiremovecallback *)dispatch_data;
 			callbacks.saAmfCSIRemoveCallback (
 				res_lib_amf_csiremovecallback->invocation,
 				&res_lib_amf_csiremovecallback->compName,
@@ -315,7 +297,7 @@ saAmfDispatch (
 			break;
 
 		case MESSAGE_RES_AMF_COMPONENTTERMINATECALLBACK:
-			res_lib_amf_componentterminatecallback = (struct res_lib_amf_componentterminatecallback *)&dispatch_data;
+			res_lib_amf_componentterminatecallback = (struct res_lib_amf_componentterminatecallback *)dispatch_data;
 			callbacks.saAmfComponentTerminateCallback (
 				res_lib_amf_componentterminatecallback->invocation,
 				&res_lib_amf_componentterminatecallback->compName);
@@ -323,7 +305,7 @@ saAmfDispatch (
 
 #ifdef COMPILE_OUT
 		case MESSAGE_RES_AMF_PROTECTIONGROUPTRACKCALLBACK:
-			res_lib_amf_protectiongrouptrackcallback = (struct res_lib_amf_protectiongrouptrackcallback *)&dispatch_data;
+			res_lib_amf_protectiongrouptrackcallback = (struct res_lib_amf_protectiongrouptrackcallback *)dispatch_data;
 			memcpy (res_lib_amf_protectiongrouptrackcallback->notificationBufferAddress,
 				res_lib_amf_protectiongrouptrackcallback->notificationBuffer,
 				res_lib_amf_protectiongrouptrackcallback->numberOfItems * sizeof (SaAmfProtectionGroupNotificationT));
@@ -336,10 +318,12 @@ saAmfDispatch (
 #endif
 			break;
 		default:
-			error = SA_AIS_ERR_LIBRARY;	
+			coroipcc_dispatch_put (amfInstance->handle);
+			error = SA_AIS_ERR_LIBRARY;
 			goto error_put;
 			break;
 		}
+		coroipcc_dispatch_put (amfInstance->handle);
 
 		/*
 		 * Determine if more messages should be processed
@@ -355,11 +339,9 @@ saAmfDispatch (
 		}
 	} while (cont);
 
-error_unlock:
-	pthread_mutex_unlock (&amfInstance->dispatch_mutex);
 error_put:
-	saHandleInstancePut (&amfHandleDatabase, amfHandle);
-
+	hdb_handle_put (&amfHandleDatabase, amfHandle);
+error_exit:
 	return (error);
 }
 
@@ -370,36 +352,26 @@ saAmfFinalize (
 	struct amfInstance *amfInstance;
 	SaAisErrorT error;
 
-	error = saHandleInstanceGet (&amfHandleDatabase, amfHandle, (void *)&amfInstance);
+	error = hdb_error_to_sa(hdb_handle_get (&amfHandleDatabase, amfHandle, (void *)&amfInstance));
 	if (error != SA_AIS_OK) {
 		return (error);
 	}
 
-	pthread_mutex_lock (&amfInstance->dispatch_mutex);
-
-	pthread_mutex_lock (&amfInstance->response_mutex);
-
 	/*
 	 * Another thread has already started finalizing
 	 */
 	if (amfInstance->finalize) {
-		pthread_mutex_unlock (&amfInstance->response_mutex);
-		pthread_mutex_unlock (&amfInstance->dispatch_mutex);
-		saHandleInstancePut (&amfHandleDatabase, amfHandle);
+		hdb_handle_put (&amfHandleDatabase, amfHandle);
 		return (SA_AIS_ERR_BAD_HANDLE);
 	}
 
 	amfInstance->finalize = 1;
 
-	coroipcc_service_disconnect (amfInstance->ipc_ctx);
+	coroipcc_service_disconnect (amfInstance->handle);
 
-	pthread_mutex_unlock (&amfInstance->response_mutex);
+	hdb_handle_destroy (&amfHandleDatabase, amfHandle);
 
-	pthread_mutex_unlock (&amfInstance->dispatch_mutex);
-
-	saHandleDestroy (&amfHandleDatabase, amfHandle);
-
-	saHandleInstancePut (&amfHandleDatabase, amfHandle);
+	hdb_handle_put (&amfHandleDatabase, amfHandle);
 
 	return (error);
 }
@@ -416,8 +388,8 @@ saAmfComponentRegister (
 	struct res_lib_amf_componentregister res_lib_amf_componentregister;
 	struct iovec iov;
 
-	error = saHandleInstanceGet (&amfHandleDatabase, amfHandle,
-		(void *)&amfInstance);
+	error = hdb_error_to_sa(hdb_handle_get (&amfHandleDatabase, amfHandle,
+		(void *)&amfInstance));
 	if (error != SA_AIS_OK) {
 		return (error);
 	}
@@ -434,19 +406,16 @@ saAmfComponentRegister (
 			sizeof (SaNameT));
 	}
 
-	iov.iov_base = &req_lib_amf_componentregister;
+	iov.iov_base = (void *)&req_lib_amf_componentregister;
 	iov.iov_len = sizeof (struct req_lib_amf_componentregister);
-	pthread_mutex_lock (&amfInstance->response_mutex);
 
-	error = coroipcc_msg_send_reply_receive (amfInstance->ipc_ctx,
+	error = coroipcc_msg_send_reply_receive (amfInstance->handle,
 		&iov,
 		1,
 		&res_lib_amf_componentregister,
 		sizeof (struct res_lib_amf_componentregister));
 
-	pthread_mutex_unlock (&amfInstance->response_mutex);
-
-	saHandleInstancePut (&amfHandleDatabase, amfHandle);
+	hdb_handle_put (&amfHandleDatabase, amfHandle);
 
 	if (res_lib_amf_componentregister.header.error == SA_AIS_OK) {
 		amfInstance->compRegistered = 1;
@@ -467,8 +436,8 @@ saAmfComponentUnregister (
 	SaAisErrorT error;
 	struct iovec iov;
 
-	error = saHandleInstanceGet (&amfHandleDatabase, amfHandle,
-		(void *)&amfInstance);
+	error = hdb_error_to_sa(hdb_handle_get (&amfHandleDatabase, amfHandle,
+		(void *)&amfInstance));
 	if (error != SA_AIS_OK) {
 		return (error);
 	}
@@ -483,22 +452,18 @@ saAmfComponentUnregister (
 	} else {
 		memset (&req_lib_amf_componentunregister.proxyCompName, 0,
 			sizeof (SaNameT));
-	}	
+	}
 
-	iov.iov_base = &req_lib_amf_componentunregister;
+	iov.iov_base = (void *)&req_lib_amf_componentunregister;
 	iov.iov_len = sizeof (struct req_lib_amf_componentunregister);
 
-	pthread_mutex_lock (&amfInstance->response_mutex);
-
-	error = coroipcc_msg_send_reply_receive (amfInstance->ipc_ctx,
+	error = coroipcc_msg_send_reply_receive (amfInstance->handle,
 		&iov,
 		1,
 		&res_lib_amf_componentunregister,
 		sizeof (struct res_lib_amf_componentunregister));
 
-	pthread_mutex_unlock (&amfInstance->response_mutex);
-
-	saHandleInstancePut (&amfHandleDatabase, amfHandle);
+	hdb_handle_put (&amfHandleDatabase, amfHandle);
 
         return (error == SA_AIS_OK ? res_lib_amf_componentunregister.header.error : error);
 }
@@ -512,14 +477,12 @@ saAmfComponentNameGet (
 	SaAisErrorT error;
 	char *env_value;
 
-	error = saHandleInstanceGet (&amfHandleDatabase, amfHandle,
-		(void *)&amfInstance);
+	error = hdb_error_to_sa(hdb_handle_get (&amfHandleDatabase, amfHandle,
+		(void *)&amfInstance));
 	if (error != SA_AIS_OK) {
 		return (error);
 	}
 
-	pthread_mutex_lock (&amfInstance->response_mutex);
-
 	error = SA_AIS_OK;
 
 	env_value = getenv ("SA_AMF_COMPONENT_NAME");
@@ -533,9 +496,7 @@ saAmfComponentNameGet (
 	compName->length = strlen (env_value);
 
 error_exit:
-	pthread_mutex_unlock (&amfInstance->response_mutex);
-
-	saHandleInstancePut (&amfHandleDatabase, amfHandle);
+	hdb_handle_put (&amfHandleDatabase, amfHandle);
 
 	return (error);
 }
@@ -555,8 +516,8 @@ saAmfPmStart (
 	SaAisErrorT error;
 	struct iovec iov;
 
-	error = saHandleInstanceGet (&amfHandleDatabase, amfHandle,
-		(void *)&amfInstance);
+	error = hdb_error_to_sa(hdb_handle_get (&amfHandleDatabase, amfHandle,
+		(void *)&amfInstance));
 	if (error != SA_AIS_OK) {
 		return (error);
 	}
@@ -570,20 +531,16 @@ saAmfPmStart (
 	req_lib_amf_pmstart.pmErrors = pmErrors;
 	req_lib_amf_pmstart.recommendedRecovery = recommendedRecovery;
 
-	iov.iov_base = &req_lib_amf_pmstart;
+	iov.iov_base = (void *)&req_lib_amf_pmstart;
 	iov.iov_len = sizeof (struct req_lib_amf_pmstart);
 
-	pthread_mutex_lock (&amfInstance->response_mutex);
-
-	error = coroipcc_msg_send_reply_receive (amfInstance->ipc_ctx,
+	error = coroipcc_msg_send_reply_receive (amfInstance->handle,
 		&iov,
 		1,
 		&res_lib_amf_pmstart,
 		sizeof (struct res_lib_amf_pmstart));
 
-	pthread_mutex_unlock (&amfInstance->response_mutex);
-
-	saHandleInstancePut (&amfHandleDatabase, amfHandle);
+	hdb_handle_put (&amfHandleDatabase, amfHandle);
 
         return (error == SA_AIS_OK ? res_lib_amf_pmstart.header.error : error);
 }
@@ -602,8 +559,8 @@ saAmfPmStop (
 	SaAisErrorT error;
 	struct iovec iov;
 
-	error = saHandleInstanceGet (&amfHandleDatabase, amfHandle,
-		(void *)&amfInstance);
+	error = hdb_error_to_sa(hdb_handle_get (&amfHandleDatabase, amfHandle,
+		(void *)&amfInstance));
 	if (error != SA_AIS_OK) {
 		return (error);
 	}
@@ -615,20 +572,16 @@ saAmfPmStop (
 	req_lib_amf_pmstop.processId = processId;
 	req_lib_amf_pmstop.pmErrors = pmErrors;
 
-	iov.iov_base = &req_lib_amf_pmstop;
+	iov.iov_base = (void *)&req_lib_amf_pmstop;
 	iov.iov_len = sizeof (struct req_lib_amf_pmstop);
 
-	pthread_mutex_lock (&amfInstance->response_mutex);
-
-	error = coroipcc_msg_send_reply_receive (amfInstance->ipc_ctx,
+	error = coroipcc_msg_send_reply_receive (amfInstance->handle,
 		&iov,
 		1,
 		&res_lib_amf_pmstop,
 		sizeof (struct res_lib_amf_pmstop));
 
-	pthread_mutex_unlock (&amfInstance->response_mutex);
-
-	saHandleInstancePut (&amfHandleDatabase, amfHandle);
+	hdb_handle_put (&amfHandleDatabase, amfHandle);
 
         return (error == SA_AIS_OK ? res_lib_amf_pmstop.header.error : error);
 	return (SA_AIS_OK);
@@ -648,8 +601,8 @@ saAmfHealthcheckStart (
 	SaAisErrorT error;
 	struct iovec iov;
 
-	error = saHandleInstanceGet (&amfHandleDatabase, amfHandle,
-		(void *)&amfInstance);
+	error = hdb_error_to_sa(hdb_handle_get (&amfHandleDatabase, amfHandle,
+		(void *)&amfInstance));
 	if (error != SA_AIS_OK) {
 		return (error);
 	}
@@ -663,20 +616,16 @@ saAmfHealthcheckStart (
 	req_lib_amf_healthcheckstart.invocationType = invocationType;
 	req_lib_amf_healthcheckstart.recommendedRecovery = recommendedRecovery;
 
-	iov.iov_base = &req_lib_amf_healthcheckstart;
+	iov.iov_base = (void *)&req_lib_amf_healthcheckstart;
 	iov.iov_len = sizeof (struct req_lib_amf_healthcheckstart);
 
-	pthread_mutex_lock (&amfInstance->response_mutex);
-
-	error = coroipcc_msg_send_reply_receive (amfInstance->ipc_ctx,
+	error = coroipcc_msg_send_reply_receive (amfInstance->handle,
 		&iov,
 		1,
 		&res_lib_amf_healthcheckstart,
 		sizeof (struct res_lib_amf_healthcheckstart));
 
-	pthread_mutex_unlock (&amfInstance->response_mutex);
-
-	saHandleInstancePut (&amfHandleDatabase, amfHandle);
+	hdb_handle_put (&amfHandleDatabase, amfHandle);
 
         return (error == SA_AIS_OK ? res_lib_amf_healthcheckstart.header.error : error);
 }
@@ -694,8 +643,8 @@ saAmfHealthcheckConfirm (
 	SaAisErrorT error;
 	struct iovec iov;
 
-	error = saHandleInstanceGet (&amfHandleDatabase, amfHandle,
-		(void *)&amfInstance);
+	error = hdb_error_to_sa(hdb_handle_get (&amfHandleDatabase, amfHandle,
+		(void *)&amfInstance));
 	if (error != SA_AIS_OK) {
 		return (error);
 	}
@@ -708,20 +657,16 @@ saAmfHealthcheckConfirm (
 		healthcheckKey, sizeof (SaAmfHealthcheckKeyT));
 	req_lib_amf_healthcheckconfirm.healthcheckResult = healthcheckResult;
 
-	iov.iov_base = &req_lib_amf_healthcheckconfirm;
+	iov.iov_base = (void *)&req_lib_amf_healthcheckconfirm;
 	iov.iov_len = sizeof (struct req_lib_amf_healthcheckconfirm);
 
-	pthread_mutex_lock (&amfInstance->response_mutex);
-
-	error = coroipcc_msg_send_reply_receive (amfInstance->ipc_ctx,
+	error = coroipcc_msg_send_reply_receive (amfInstance->handle,
 		&iov,
 		1,
 		&res_lib_amf_healthcheckconfirm,
 		sizeof (struct res_lib_amf_healthcheckconfirm));
 
-	pthread_mutex_unlock (&amfInstance->response_mutex);
-
-	saHandleInstancePut (&amfHandleDatabase, amfHandle);
+	hdb_handle_put (&amfHandleDatabase, amfHandle);
 
         return (error == SA_AIS_OK ? res_lib_amf_healthcheckconfirm.header.error : error);
 }
@@ -738,8 +683,8 @@ saAmfHealthcheckStop (
 	struct iovec iov;
 	SaAisErrorT error;
 
-	error = saHandleInstanceGet (&amfHandleDatabase, amfHandle,
-		(void *)&amfInstance);
+	error = hdb_error_to_sa(hdb_handle_get (&amfHandleDatabase, amfHandle,
+		(void *)&amfInstance));
 	if (error != SA_AIS_OK) {
 		return (error);
 	}
@@ -751,20 +696,16 @@ saAmfHealthcheckStop (
 	memcpy (&req_lib_amf_healthcheckstop.healthcheckKey,
 		healthcheckKey, sizeof (SaAmfHealthcheckKeyT));
 
-	iov.iov_base = &req_lib_amf_healthcheckstop;
+	iov.iov_base = (void *)&req_lib_amf_healthcheckstop;
 	iov.iov_len = sizeof (struct req_lib_amf_healthcheckstop);
 
-	pthread_mutex_lock (&amfInstance->response_mutex);
-
-	error = coroipcc_msg_send_reply_receive (amfInstance->ipc_ctx,
+	error = coroipcc_msg_send_reply_receive (amfInstance->handle,
 		&iov,
 		1,
 		&res_lib_amf_healthcheckstop,
 		sizeof (struct res_lib_amf_healthcheckstop));
 
-	pthread_mutex_unlock (&amfInstance->response_mutex);
-
-	saHandleInstancePut (&amfHandleDatabase, amfHandle);
+	hdb_handle_put (&amfHandleDatabase, amfHandle);
 
         return (error == SA_AIS_OK ? res_lib_amf_healthcheckstop.header.error : error);
 }
@@ -783,31 +724,27 @@ saAmfHAStateGet (
 	struct iovec iov;
 	SaAisErrorT error;
 
-	error = saHandleInstanceGet (&amfHandleDatabase, amfHandle,
-		(void *)&amfInstance);
+	error = hdb_error_to_sa(hdb_handle_get (&amfHandleDatabase, amfHandle,
+		(void *)&amfInstance));
 	if (error != SA_AIS_OK) {
 		return (error);
 	}
 
-	iov.iov_base = &req_lib_amf_hastateget,
+	iov.iov_base = (void *)&req_lib_amf_hastateget,
 	iov.iov_len = sizeof (struct req_lib_amf_hastateget),
 
-	pthread_mutex_lock (&amfInstance->response_mutex);
-
 	req_lib_amf_hastateget.header.id = MESSAGE_REQ_AMF_HASTATEGET;
 	req_lib_amf_hastateget.header.size = sizeof (struct req_lib_amf_hastateget);
 	memcpy (&req_lib_amf_hastateget.compName, compName, sizeof (SaNameT));
 	memcpy (&req_lib_amf_hastateget.csiName, csiName, sizeof (SaNameT));
 
-	error = coroipcc_msg_send_reply_receive (amfInstance->ipc_ctx,
+	error = coroipcc_msg_send_reply_receive (amfInstance->handle,
 		&iov,
 		1,
 		&res_lib_amf_hastateget,
 		sizeof (struct res_lib_amf_hastateget));
 
-	pthread_mutex_unlock (&amfInstance->response_mutex);
-
-	saHandleInstancePut (&amfHandleDatabase, amfHandle);
+	hdb_handle_put (&amfHandleDatabase, amfHandle);
 
 	if (res_lib_amf_hastateget.header.error == SA_AIS_OK) {
 		memcpy (haState, &res_lib_amf_hastateget.haState,
@@ -828,8 +765,8 @@ saAmfCSIQuiescingComplete (
 	struct iovec iov;
 	SaAisErrorT errorResult;
 
-	error = saHandleInstanceGet (&amfHandleDatabase, amfHandle,
-		(void *)&amfInstance);
+	error = hdb_error_to_sa(hdb_handle_get (&amfHandleDatabase, amfHandle,
+		(void *)&amfInstance));
 	if (error != SA_AIS_OK) {
 		return (error);
 	}
@@ -839,20 +776,16 @@ saAmfCSIQuiescingComplete (
 	req_lib_amf_csiquiescingcomplete.invocation = invocation;
 	req_lib_amf_csiquiescingcomplete.error = error;
 
-	iov.iov_base = &req_lib_amf_csiquiescingcomplete;
+	iov.iov_base = (void *)&req_lib_amf_csiquiescingcomplete;
 	iov.iov_len = sizeof (struct req_lib_amf_csiquiescingcomplete);
 
-	pthread_mutex_lock (&amfInstance->response_mutex);
-
-	errorResult = coroipcc_msg_send_reply_receive (amfInstance->ipc_ctx,
+	errorResult = coroipcc_msg_send_reply_receive (amfInstance->handle,
 		&iov,
 		1,
 		&res_lib_amf_csiquiescingcomplete,
 		sizeof (struct res_lib_amf_csiquiescingcomplete));
 
-	pthread_mutex_unlock (&amfInstance->response_mutex);
-
-	saHandleInstancePut (&amfHandleDatabase, amfHandle);
+	hdb_handle_put (&amfHandleDatabase, amfHandle);
 
         return (errorResult == SA_AIS_OK ? res_lib_amf_csiquiescingcomplete.header.error : errorResult);
 }
@@ -877,26 +810,22 @@ saAmfProtectionGroupTrack (
 	req_lib_amf_protectiongrouptrack.trackFlags = trackFlags;
 	req_lib_amf_protectiongrouptrack.notificationBufferAddress = (SaAmfProtectionGroupNotificationBufferT *)notificationBuffer;
 
-	error = saHandleInstanceGet (&amfHandleDatabase, amfHandle,
-		(void *)&amfInstance);
+	error = hdb_error_to_sa(hdb_handle_get (&amfHandleDatabase, amfHandle,
+		(void *)&amfInstance));
 	if (error != SA_AIS_OK) {
 		return (error);
 	}
 
-	iov.iov_base = &req_lib_amf_protectiongrouptrack;
+	iov.iov_base = (void *)&req_lib_amf_protectiongrouptrack;
 	iov.iov_len = sizeof (struct req_lib_amf_protectiongrouptrack);
 
-	pthread_mutex_lock (&amfInstance->response_mutex);
-
-	error = coroipcc_msg_send_reply_receive (amfInstance->ipc_ctx,
+	error = coroipcc_msg_send_reply_receive (amfInstance->handle,
 		&iov,
 		1,
 		&res_lib_amf_protectiongrouptrack,
 		sizeof (struct res_lib_amf_protectiongrouptrack));
 
-	pthread_mutex_unlock (&amfInstance->response_mutex);
-
-	saHandleInstancePut (&amfHandleDatabase, amfHandle);
+	hdb_handle_put (&amfHandleDatabase, amfHandle);
 
         return (error == SA_AIS_OK ? res_lib_amf_protectiongrouptrack.header.error : error);
 }
@@ -912,8 +841,8 @@ saAmfProtectionGroupTrackStop (
 	struct iovec iov;
 	SaAisErrorT error;
 
-	error = saHandleInstanceGet (&amfHandleDatabase, amfHandle,
-		(void *)&amfInstance);
+	error = hdb_error_to_sa(hdb_handle_get (&amfHandleDatabase, amfHandle,
+		(void *)&amfInstance));
 	if (error != SA_AIS_OK) {
 		return (error);
 	}
@@ -922,19 +851,16 @@ saAmfProtectionGroupTrackStop (
 	req_lib_amf_protectiongrouptrackstop.header.id = MESSAGE_REQ_AMF_PROTECTIONGROUPTRACKSTOP;
 	memcpy (&req_lib_amf_protectiongrouptrackstop.csiName, csiName, sizeof (SaNameT));
 
-	iov.iov_base = &req_lib_amf_protectiongrouptrackstop,
+	iov.iov_base = (void *)&req_lib_amf_protectiongrouptrackstop,
 	iov.iov_len = sizeof (struct req_lib_amf_protectiongrouptrackstop),
-	pthread_mutex_lock (&amfInstance->response_mutex);
 
-	error = coroipcc_msg_send_reply_receive (amfInstance->ipc_ctx,
+	error = coroipcc_msg_send_reply_receive (amfInstance->handle,
 		&iov,
 		1,
 		&res_lib_amf_protectiongrouptrackstop,
 		sizeof (struct res_lib_amf_protectiongrouptrackstop));
 
-	pthread_mutex_unlock (&amfInstance->response_mutex);
-
-	saHandleInstancePut (&amfHandleDatabase, amfHandle);
+	hdb_handle_put (&amfHandleDatabase, amfHandle);
 
         return (error == SA_AIS_OK ? res_lib_amf_protectiongrouptrackstop.header.error : error);
 }
@@ -953,8 +879,8 @@ saAmfComponentErrorReport (
 	struct iovec iov;
 	SaAisErrorT error;
 
-	error = saHandleInstanceGet (&amfHandleDatabase, amfHandle,
-		(void *)&amfInstance);
+	error = hdb_error_to_sa(hdb_handle_get (&amfHandleDatabase, amfHandle,
+		(void *)&amfInstance));
 	if (error != SA_AIS_OK) {
 		return (error);
 	}
@@ -966,10 +892,10 @@ saAmfComponentErrorReport (
 	req_lib_amf_componenterrorreport.errorDetectionTime = errorDetectionTime;
 	req_lib_amf_componenterrorreport.recommendedRecovery = recommendedRecovery;
 
-	iov.iov_base = &req_lib_amf_componenterrorreport;
+	iov.iov_base = (void *)&req_lib_amf_componenterrorreport;
 	iov.iov_len = sizeof (struct req_lib_amf_componenterrorreport);
 
-	error = coroipcc_msg_send_reply_receive (amfInstance->ipc_ctx,
+	error = coroipcc_msg_send_reply_receive (amfInstance->handle,
 		&iov,
 		1,
 		&res_lib_amf_componenterrorreport,
@@ -978,9 +904,7 @@ saAmfComponentErrorReport (
 
 	error = res_lib_amf_componenterrorreport.header.error;
 
-	pthread_mutex_unlock (&amfInstance->response_mutex);
-
-	saHandleInstancePut (&amfHandleDatabase, amfHandle);
+	hdb_handle_put (&amfHandleDatabase, amfHandle);
 
         return (error == SA_AIS_OK ? res_lib_amf_componenterrorreport.header.error : error);
 }
@@ -997,8 +921,8 @@ saAmfComponentErrorClear (
 	struct iovec iov;
 	SaAisErrorT error;
 
-	error = saHandleInstanceGet (&amfHandleDatabase, amfHandle,
-		(void *)&amfInstance);
+	error = hdb_error_to_sa(hdb_handle_get (&amfHandleDatabase, amfHandle,
+		(void *)&amfInstance));
 	if (error != SA_AIS_OK) {
 		return (error);
 	}
@@ -1007,20 +931,16 @@ saAmfComponentErrorClear (
 	req_lib_amf_componenterrorclear.header.size = sizeof (struct req_lib_amf_componenterrorclear);
 	memcpy (&req_lib_amf_componenterrorclear.compName, compName, sizeof (SaNameT));
 
-	iov.iov_base = &req_lib_amf_componenterrorclear;
+	iov.iov_base = (void *)&req_lib_amf_componenterrorclear;
 	iov.iov_len = sizeof (struct req_lib_amf_componenterrorclear);
 
-	pthread_mutex_lock (&amfInstance->response_mutex);
-
-	error = coroipcc_msg_send_reply_receive (amfInstance->ipc_ctx,
+	error = coroipcc_msg_send_reply_receive (amfInstance->handle,
 		&iov,
 		1,
 		&res_lib_amf_componenterrorclear,
 		sizeof (struct res_lib_amf_componenterrorclear));
 
-	pthread_mutex_unlock (&amfInstance->response_mutex);
-
-	saHandleInstancePut (&amfHandleDatabase, amfHandle);
+	hdb_handle_put (&amfHandleDatabase, amfHandle);
 
         return (error == SA_AIS_OK ? res_lib_amf_componenterrorclear.header.error : error);
 }
@@ -1037,8 +957,8 @@ saAmfResponse (
 	struct iovec iov;
 	SaAisErrorT errorResult;
 
-	errorResult = saHandleInstanceGet (&amfHandleDatabase, amfHandle,
-		(void *)&amfInstance);
+	errorResult = hdb_error_to_sa(hdb_handle_get (&amfHandleDatabase, amfHandle,
+		(void *)&amfInstance));
 	if (errorResult != SA_AIS_OK) {
 		return (errorResult);
 	}
@@ -1048,20 +968,17 @@ saAmfResponse (
 	req_lib_amf_response.invocation = invocation;
 	req_lib_amf_response.error = error;
 
-	iov.iov_base = &req_lib_amf_response;
+	iov.iov_base = (void *)&req_lib_amf_response;
 	iov.iov_len = sizeof (struct req_lib_amf_response);
 
-	pthread_mutex_lock (&amfInstance->response_mutex);
-
-	errorResult = coroipcc_msg_send_reply_receive (amfInstance->ipc_ctx,
+	errorResult = coroipcc_msg_send_reply_receive (
+		amfInstance->handle,
 		&iov,
 		1,
 		&res_lib_amf_response,
 		sizeof (struct res_lib_amf_response));
 
-	pthread_mutex_unlock (&amfInstance->response_mutex);
-
-	saHandleInstancePut (&amfHandleDatabase, amfHandle);
+	hdb_handle_put (&amfHandleDatabase, amfHandle);
 
         return (errorResult == SA_AIS_OK ? res_lib_amf_response.header.error : errorResult);
 }
