@@ -1,13 +1,13 @@
 /*
  * Copyright (c) 2002-2005 MontaVista Software, Inc.
- * Copyright (c) 2006 Red Hat, Inc.
+ * Copyright (c) 2006, 2009 Red Hat, Inc.
  *
  * All rights reserved.
  *
  * Author: Steven Dake (sdake@redhat.com)
  *
  * This software licensed under BSD license, the text of which follows:
- * 
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
  *
@@ -33,11 +33,12 @@
  * THE POSSIBILITY OF SUCH DAMAGE.
  */
 
+#include <config.h>
+
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
 #include <unistd.h>
-#include <assert.h>
 #include <errno.h>
 #include <pthread.h>
 #include <sys/types.h>
@@ -46,38 +47,27 @@
 #include <sys/un.h>
 
 #define PROCESSOR_COUNT_MAX 384
+#include <corosync/corotypes.h>
+#include <corosync/coroipc_types.h>
+#include <corosync/coroipcc.h>
+#include <corosync/corodefs.h>
+#include <corosync/hdb.h>
 #include <corosync/swab.h>
-#include <corosync/ipc_gen.h>
 #include <corosync/mar_gen.h>
 #include <saAis.h>
 #include <saClm.h>
 #include <ipc_clm.h>
-#include <corosync/coroipcc.h>
 #include "../include/mar_clm.h"
 
 #include "util.h"
 
-struct res_overlay {
-	mar_res_header_t header;
-	char data[512000];
-};
-
 struct clmInstance {
-	void *ipc_ctx;
+	hdb_handle_t ipc_handle;
 	SaClmCallbacksT callbacks;
 	int finalize;
-	pthread_mutex_t response_mutex;
-	pthread_mutex_t dispatch_mutex;
 };
 
-static void clmHandleInstanceDestructor (void *);
-
-static struct saHandleDatabase clmHandleDatabase = {
-	.handleCount			= 0,
-	.handles			= 0,
-	.mutex				= PTHREAD_MUTEX_INITIALIZER,
-	.handleInstanceDestructor	= clmHandleInstanceDestructor
-};
+DECLARE_HDB_DATABASE(clmHandleDatabase,NULL);
 
 /*
  * Versions supported
@@ -91,15 +81,6 @@ static struct saVersionDatabase clmVersionDatabase = {
 	clmVersionsSupported
 };
 
-void clmHandleInstanceDestructor (void *instance)
-{
-	struct clmInstance *clmInstance = instance;
-
-	pthread_mutex_destroy (&clmInstance->response_mutex);
-	pthread_mutex_destroy (&clmInstance->dispatch_mutex);
-}
-
-
 /**
  * @defgroup saClm SAF AIS Cluster Membership API
  * @ingroup saf
@@ -159,19 +140,25 @@ saClmInitialize (
 		goto error_no_destroy;
 	}
 
-	error = saHandleCreate (&clmHandleDatabase, sizeof (struct clmInstance),
-		clmHandle);
+	error = hdb_error_to_sa(hdb_handle_create (&clmHandleDatabase, sizeof (struct clmInstance),
+		clmHandle));
 	if (error != SA_AIS_OK) {
 		goto error_no_destroy;
 	}
 
-	error = saHandleInstanceGet (&clmHandleDatabase, *clmHandle,
-		(void *)&clmInstance);
+	error = hdb_error_to_sa(hdb_handle_get (&clmHandleDatabase, *clmHandle,
+		(void *)&clmInstance));
 	if (error != SA_AIS_OK) {
 		goto error_destroy;
 	}
 
-	error = coroipcc_service_connect (IPC_SOCKET_NAME, CLM_SERVICE, &clmInstance->ipc_ctx);
+	error = coroipcc_service_connect (
+		COROSYNC_SOCKET_NAME,
+		CLM_SERVICE,
+		IPC_REQUEST_SIZE,
+		IPC_RESPONSE_SIZE,
+		IPC_DISPATCH_SIZE,
+		&clmInstance->ipc_handle);
 	if (error != SA_AIS_OK) {
 		goto error_put_destroy;
 	}
@@ -182,18 +169,14 @@ saClmInitialize (
 		memset (&clmInstance->callbacks, 0, sizeof (SaClmCallbacksT));
 	}
 
-	pthread_mutex_init (&clmInstance->response_mutex, NULL);
-
-	pthread_mutex_init (&clmInstance->dispatch_mutex, NULL);
-
-	saHandleInstancePut (&clmHandleDatabase, *clmHandle);
+	hdb_handle_put (&clmHandleDatabase, *clmHandle);
 
 	return (SA_AIS_OK);
 
 error_put_destroy:
-	saHandleInstancePut (&clmHandleDatabase, *clmHandle);
+	hdb_handle_put (&clmHandleDatabase, *clmHandle);
 error_destroy:
-	saHandleDestroy (&clmHandleDatabase, *clmHandle);
+	hdb_handle_destroy (&clmHandleDatabase, *clmHandle);
 error_no_destroy:
 	return (error);
 }
@@ -227,20 +210,22 @@ saClmSelectionObjectGet (
 {
 	struct clmInstance *clmInstance;
 	SaAisErrorT error;
+	int fd;
 
 	if (selectionObject == NULL) {
 		return (SA_AIS_ERR_INVALID_PARAM);
 	}
-	error = saHandleInstanceGet (&clmHandleDatabase, clmHandle,
-		(void *)&clmInstance);
+	error = hdb_error_to_sa(hdb_handle_get (&clmHandleDatabase, clmHandle,
+		(void *)&clmInstance));
 	if (error != SA_AIS_OK) {
 		return (error);
 	}
 
-	*selectionObject = coroipcc_fd_get (clmInstance->ipc_ctx);
+	error = coroipcc_fd_get (clmInstance->ipc_handle, &fd);
+	*selectionObject = fd;
 
-	saHandleInstancePut (&clmHandleDatabase, clmHandle);
-	return (SA_AIS_OK);
+	hdb_handle_put (&clmHandleDatabase, clmHandle);
+	return (error);
 }
 
 /**
@@ -267,12 +252,11 @@ saClmDispatch (
 	int timeout = -1;
 	SaAisErrorT error;
 	int cont = 1; /* always continue do loop except when set to 0 */
-	int dispatch_avail;
 	struct clmInstance *clmInstance;
 	struct res_lib_clm_clustertrack *res_lib_clm_clustertrack;
 	struct res_clm_nodegetcallback *res_clm_nodegetcallback;
 	SaClmCallbacksT callbacks;
-	struct res_overlay dispatch_data;
+	coroipc_response_header_t *dispatch_data;
 	SaClmClusterNotificationBufferT notificationBuffer;
 	SaClmClusterNotificationT notification[PROCESSOR_COUNT_MAX];
 	SaClmClusterNodeT clusterNode;
@@ -286,8 +270,8 @@ saClmDispatch (
 		return (SA_AIS_ERR_INVALID_PARAM);
 	}
 
-	error = saHandleInstanceGet (&clmHandleDatabase, clmHandle,
-		(void *)&clmInstance);
+	error = hdb_error_to_sa(hdb_handle_get (&clmHandleDatabase, clmHandle,
+		(void *)&clmInstance));
 	if (error != SA_AIS_OK) {
 		return (error);
 	}
@@ -301,27 +285,24 @@ saClmDispatch (
 	}
 
 	do {
-		pthread_mutex_lock (&clmInstance->dispatch_mutex);
-
-		dispatch_avail = coroipcc_dispatch_recv (clmInstance->ipc_ctx,
-			(void *)&dispatch_data, timeout);
-
-		pthread_mutex_unlock (&clmInstance->dispatch_mutex);
-
-		if (dispatch_avail == 0 && dispatchFlags == SA_DISPATCH_ALL) {
-			break; /* exit do while cont is 1 loop */
-		} else
-		if (dispatch_avail == 0) {
-			continue; /* next poll */
+		error = coroipcc_dispatch_get (
+			clmInstance->ipc_handle,
+			(void **)&dispatch_data,
+			timeout);
+		if (error == CS_ERR_BAD_HANDLE) {
+			error = CS_OK;
+			goto error_put;
+		}
+		if (error != CS_OK) {
+			goto error_put;
 		}
 
-		if (dispatch_avail == -1) {
-			if (clmInstance->finalize == 1) {
-				error = SA_AIS_OK;
+		if (dispatch_data == NULL) {
+			if (dispatchFlags == CPG_DISPATCH_ALL) {
+				break; /* exit do while cont is 1 loop */
 			} else {
-				error = SA_AIS_ERR_LIBRARY;
+				continue; /* next poll */
 			}
-			goto error_put;
 		}
 
 		/*
@@ -334,13 +315,13 @@ saClmDispatch (
 		/*
 		 * Dispatch incoming message
 		 */
-		switch (dispatch_data.header.id) {
+		switch (dispatch_data->id) {
 
 		case MESSAGE_RES_CLM_TRACKCALLBACK:
 			if (callbacks.saClmClusterTrackCallback == NULL) {
 				continue;
 			}
-			res_lib_clm_clustertrack = (struct res_lib_clm_clustertrack *)&dispatch_data;
+			res_lib_clm_clustertrack = (struct res_lib_clm_clustertrack *)dispatch_data;
 			error = SA_AIS_OK;
 
 			notificationBuffer.notification = notification;
@@ -371,7 +352,7 @@ saClmDispatch (
 			if (callbacks.saClmClusterNodeGetCallback == NULL) {
 				continue;
 			}
-			res_clm_nodegetcallback = (struct res_clm_nodegetcallback *)&dispatch_data;
+			res_clm_nodegetcallback = (struct res_clm_nodegetcallback *)dispatch_data;
 			marshall_from_mar_clm_cluster_node_t (
 				&clusterNode,
 				&res_clm_nodegetcallback->cluster_node);
@@ -383,10 +364,12 @@ saClmDispatch (
 			break;
 
 		default:
+			coroipcc_dispatch_put (clmInstance->ipc_handle);
 			error = SA_AIS_ERR_LIBRARY;
 			goto error_put;
 			break;
 		}
+		coroipcc_dispatch_put (clmInstance->ipc_handle);
 
 		/*
 		 * Determine if more messages should be processed
@@ -405,7 +388,7 @@ saClmDispatch (
 	goto error_put;
 
 error_put:
-	saHandleInstancePut (&clmHandleDatabase, clmHandle);
+	hdb_handle_put (&clmHandleDatabase, clmHandle);
 	return (error);
 }
 
@@ -440,32 +423,27 @@ saClmFinalize (
 	struct clmInstance *clmInstance;
 	SaAisErrorT error;
 
-	error = saHandleInstanceGet (&clmHandleDatabase, clmHandle,
-		(void *)&clmInstance);
+	error = hdb_error_to_sa(hdb_handle_get (&clmHandleDatabase, clmHandle,
+		(void *)&clmInstance));
 	if (error != SA_AIS_OK) {
 		return (error);
 	}
 
-       pthread_mutex_lock (&clmInstance->response_mutex);
-
 	/*
 	 * Another thread has already started finalizing
 	 */
 	if (clmInstance->finalize) {
-		pthread_mutex_unlock (&clmInstance->response_mutex);
-		saHandleInstancePut (&clmHandleDatabase, clmHandle);
+		hdb_handle_put (&clmHandleDatabase, clmHandle);
 		return (SA_AIS_ERR_BAD_HANDLE);
 	}
 
 	clmInstance->finalize = 1;
 
-	coroipcc_service_disconnect (clmInstance->ipc_ctx);
+	coroipcc_service_disconnect (clmInstance->ipc_handle);
 
-	pthread_mutex_unlock (&clmInstance->response_mutex);
+	hdb_handle_destroy (&clmHandleDatabase, clmHandle);
 
-	saHandleDestroy (&clmHandleDatabase, clmHandle);
-
-	saHandleInstancePut (&clmHandleDatabase, clmHandle);
+	hdb_handle_put (&clmHandleDatabase, clmHandle);
 
 	return (error);
 }
@@ -484,8 +462,8 @@ saClmClusterTrack (
 	unsigned int i;
 	struct iovec iov;
 
-	error = saHandleInstanceGet (&clmHandleDatabase, clmHandle,
-		(void *)&clmInstance);
+	error = hdb_error_to_sa(hdb_handle_get (&clmHandleDatabase, clmHandle,
+		(void *)&clmInstance));
 	if (error != SA_AIS_OK) {
 		return (error);
 	}
@@ -507,7 +485,7 @@ saClmClusterTrack (
 		error = SA_AIS_ERR_INVALID_PARAM;
 		goto error_nounlock;
 	}
-		
+
 	req_lib_clm_clustertrack.header.size = sizeof (struct req_lib_clm_clustertrack);
 	req_lib_clm_clustertrack.header.id = MESSAGE_REQ_CLM_TRACKSTART;
 	req_lib_clm_clustertrack.track_flags = trackFlags;
@@ -516,11 +494,9 @@ saClmClusterTrack (
 		req_lib_clm_clustertrack.return_in_callback = 1;
 	}
 
-	iov.iov_base = &req_lib_clm_clustertrack;
+	iov.iov_base = (void *)&req_lib_clm_clustertrack;
 	iov.iov_len = sizeof (req_lib_clm_clustertrack);;
 
-	pthread_mutex_lock (&clmInstance->response_mutex);
-
 	if ((clmInstance->callbacks.saClmClusterTrackCallback == 0) &&
 		((notificationBuffer == NULL) ||
 		(trackFlags & (SA_TRACK_CHANGES | SA_TRACK_CHANGES_ONLY)))) {
@@ -529,7 +505,7 @@ saClmClusterTrack (
 		goto error_exit;
 	}
 
-	error = coroipcc_msg_send_reply_receive (clmInstance->ipc_ctx,
+	error = coroipcc_msg_send_reply_receive (clmInstance->ipc_handle,
 		&iov,
 		1,
 		&res_lib_clm_clustertrack,
@@ -563,10 +539,9 @@ saClmClusterTrack (
 	}
 
 error_exit:
-	pthread_mutex_unlock (&clmInstance->response_mutex);
 
 error_nounlock:
-	saHandleInstancePut (&clmHandleDatabase, clmHandle);
+	hdb_handle_put (&clmHandleDatabase, clmHandle);
 
         return (error == SA_AIS_OK ? res_lib_clm_clustertrack.header.error : error);
 }
@@ -583,26 +558,22 @@ saClmClusterTrackStop (
 
 	req_lib_clm_trackstop.header.size = sizeof (struct req_lib_clm_trackstop);
 	req_lib_clm_trackstop.header.id = MESSAGE_REQ_CLM_TRACKSTOP;
-	error = saHandleInstanceGet (&clmHandleDatabase, clmHandle,
-		(void *)&clmInstance);
+	error = hdb_error_to_sa(hdb_handle_get (&clmHandleDatabase, clmHandle,
+		(void *)&clmInstance));
 	if (error != SA_AIS_OK) {
 		return (error);
 	}
 
-	iov.iov_base = &req_lib_clm_trackstop;
+	iov.iov_base = (void *)&req_lib_clm_trackstop;
 	iov.iov_len = sizeof (struct req_lib_clm_trackstop);
 
-	pthread_mutex_lock (&clmInstance->response_mutex);
-
-	error = coroipcc_msg_send_reply_receive (clmInstance->ipc_ctx,
+	error = coroipcc_msg_send_reply_receive (clmInstance->ipc_handle,
 		&iov,
 		1,
 		&res_lib_clm_trackstop,
 		sizeof (struct res_lib_clm_trackstop));
 
-	pthread_mutex_unlock (&clmInstance->response_mutex);
-
-	saHandleInstancePut (&clmHandleDatabase, clmHandle);
+	hdb_handle_put (&clmHandleDatabase, clmHandle);
 
         return (error == SA_AIS_OK ? res_lib_clm_trackstop.header.error : error);
 }
@@ -628,17 +599,15 @@ saClmClusterNodeGet (
 		return (SA_AIS_ERR_TIMEOUT);
 	}
 
-	error = saHandleInstanceGet (&clmHandleDatabase, clmHandle,
-		(void *)&clmInstance);
+	error = hdb_error_to_sa(hdb_handle_get (&clmHandleDatabase, clmHandle,
+		(void *)&clmInstance));
 	if (error != SA_AIS_OK) {
 		return (error);
 	}
 
-	iov.iov_base = &req_lib_clm_nodeget,
+	iov.iov_base = (void *)&req_lib_clm_nodeget,
 	iov.iov_len = sizeof (struct req_lib_clm_nodeget),
 
-	pthread_mutex_lock (&clmInstance->response_mutex);
-
 	/*
 	 * Send request message
 	 */
@@ -646,7 +615,7 @@ saClmClusterNodeGet (
 	req_lib_clm_nodeget.header.id = MESSAGE_REQ_CLM_NODEGET;
 	req_lib_clm_nodeget.node_id = nodeId;
 
-	error = coroipcc_msg_send_reply_receive (clmInstance->ipc_ctx,
+	error = coroipcc_msg_send_reply_receive (clmInstance->ipc_handle,
 		&iov,
 		1,
 		&res_clm_nodeget,
@@ -661,9 +630,7 @@ saClmClusterNodeGet (
 		&res_clm_nodeget.cluster_node);
 
 error_exit:
-	pthread_mutex_unlock (&clmInstance->response_mutex);
-
-	saHandleInstancePut (&clmHandleDatabase, clmHandle);
+	hdb_handle_put (&clmHandleDatabase, clmHandle);
 
 	return (error);
 }
@@ -685,24 +652,22 @@ saClmClusterNodeGetAsync (
 	req_lib_clm_nodegetasync.invocation = invocation;
 	req_lib_clm_nodegetasync.node_id = nodeId;
 
-	error = saHandleInstanceGet (&clmHandleDatabase, clmHandle,
-		(void *)&clmInstance);
+	error = hdb_error_to_sa(hdb_handle_get (&clmHandleDatabase, clmHandle,
+		(void *)&clmInstance));
 	if (error != SA_AIS_OK) {
 		return (error);
 	}
 
-	pthread_mutex_lock (&clmInstance->response_mutex);
-
 	if (clmInstance->callbacks.saClmClusterNodeGetCallback == NULL) {
 		error = SA_AIS_ERR_INIT;
 		goto error_exit;
 	}
 
-	iov.iov_base = &req_lib_clm_nodegetasync;
+	iov.iov_base = (void *)&req_lib_clm_nodegetasync;
 	iov.iov_len = sizeof (req_lib_clm_nodegetasync);
 
 	error = coroipcc_msg_send_reply_receive (
-		clmInstance->ipc_ctx,
+		clmInstance->ipc_handle,
 		&iov,
 		1,
 		&res_clm_nodegetasync,
@@ -714,9 +679,7 @@ saClmClusterNodeGetAsync (
 	error = res_clm_nodegetasync.header.error;
 
 error_exit:
-	pthread_mutex_unlock (&clmInstance->response_mutex);
-
-	saHandleInstancePut (&clmHandleDatabase, clmHandle);
+	hdb_handle_put (&clmHandleDatabase, clmHandle);
 
 	return (error);
 }
