@@ -6,7 +6,7 @@
  * Author: Ryan O'Hara (rohara@redhat.com)
  *
  * This software licensed under BSD license, the text of which follows:
- * 
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
  *
@@ -32,10 +32,11 @@
  * THE POSSIBILITY OF SUCH DAMAGE.
  */
 
+#include <config.h>
+
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
-#include <assert.h>
 #include <unistd.h>
 #include <inttypes.h>
 #include <errno.h>
@@ -50,35 +51,24 @@
 #include <saAis.h>
 #include <saTmr.h>
 
+#include <corosync/corotypes.h>
+#include <corosync/coroipc_types.h>
 #include <corosync/coroipcc.h>
+#include <corosync/corodefs.h>
+#include <corosync/hdb.h>
 #include <corosync/list.h>
-#include <corosync/ipc_gen.h>
 
 #include "../include/ipc_tmr.h"
 #include "util.h"
 
-struct message_overlay {
-	mar_res_header_t header __attribute__((aligned(8)));
-	char data[4096];
-};
-
 struct tmrInstance {
-	void *ipc_ctx;
+	hdb_handle_t ipc_handle;
 	SaTmrCallbacksT callbacks;
 	int finalize;
 	SaTmrHandleT tmrHandle;
-	pthread_mutex_t response_mutex;
-	pthread_mutex_t dispatch_mutex;
 };
 
-void tmrHandleInstanceDestructor (void *instance);
-
-static struct saHandleDatabase tmrHandleDatabase = {
-	.handleCount			= 0,
-	.handles			= 0,
-	.mutex				= PTHREAD_MUTEX_INITIALIZER,
-	.handleInstanceDestructor	= tmrHandleInstanceDestructor
-};
+DECLARE_HDB_DATABASE (tmrHandleDatabase, NULL);
 
 static SaVersionT tmrVersionsSupported[] = {
 	{ 'A', 1, 1 }
@@ -89,21 +79,6 @@ static struct saVersionDatabase tmrVersionDatabase = {
 	tmrVersionsSupported
 };
 
-void tmrHandleInstanceDestructor (void *instance)
-{
-	struct tmrInstance *tmrInstance = instance;
-
-	pthread_mutex_destroy (&tmrInstance->response_mutex);
-	pthread_mutex_destroy (&tmrInstance->dispatch_mutex);
-}
-
-#ifdef COMPILE_OUT
-static void tmrInstanceFinalize (struct tmrInstance *tmrInstance)
-{
-	return;
-}
-#endif /* COMPILE_OUT */
-
 SaAisErrorT
 saTmrInitialize (
 	SaTmrHandleT *tmrHandle,
@@ -113,29 +88,33 @@ saTmrInitialize (
 	struct tmrInstance *tmrInstance;
 	SaAisErrorT error = SA_AIS_OK;
 
-	/* DEBUG */
-	printf ("[DEBUG]: saTmrInitialize\n");
-
 	if (tmrHandle == NULL) {
-		return (SA_AIS_ERR_INVALID_PARAM);
+		error = SA_AIS_ERR_INVALID_PARAM;
+		goto error_exit;
 	}
 
 	error = saVersionVerify (&tmrVersionDatabase, version);
 	if (error != SA_AIS_OK) {
-		goto error_no_destroy;
+		goto error_exit;
 	}
 
-	error = saHandleCreate (&tmrHandleDatabase, sizeof (struct tmrInstance), tmrHandle);
+	error = hdb_error_to_sa(hdb_handle_create (&tmrHandleDatabase, sizeof (struct tmrInstance), tmrHandle));
 	if (error != SA_AIS_OK) {
-		goto error_no_destroy;
+		goto error_exit;
 	}
 
-	error = saHandleInstanceGet (&tmrHandleDatabase, *tmrHandle, (void *)&tmrInstance);
+	error = hdb_error_to_sa(hdb_handle_get (&tmrHandleDatabase, *tmrHandle, (void *)&tmrInstance));
 	if (error != SA_AIS_OK) {
 		goto error_destroy;
 	}
 
-	error = coroipcc_service_connect (IPC_SOCKET_NAME, TMR_SERVICE, &tmrInstance->ipc_ctx);
+	error = coroipcc_service_connect (
+		COROSYNC_SOCKET_NAME,
+		TMR_SERVICE,
+		IPC_REQUEST_SIZE,
+		IPC_RESPONSE_SIZE,
+		IPC_DISPATCH_SIZE,
+		&tmrInstance->ipc_handle);
 	if (error != SA_AIS_OK) {
 		goto error_put_destroy;
 	}
@@ -148,17 +127,15 @@ saTmrInitialize (
 
 	tmrInstance->tmrHandle = *tmrHandle;
 
-	pthread_mutex_init (&tmrInstance->response_mutex, NULL);
-
-	saHandleInstancePut (&tmrHandleDatabase, *tmrHandle);
+	hdb_handle_put (&tmrHandleDatabase, *tmrHandle);
 
 	return (SA_AIS_OK);
 
 error_put_destroy:
-	saHandleInstancePut (&tmrHandleDatabase, *tmrHandle);
+	hdb_handle_put (&tmrHandleDatabase, *tmrHandle);
 error_destroy:
-	saHandleDestroy (&tmrHandleDatabase, *tmrHandle);
-error_no_destroy:
+	hdb_handle_destroy (&tmrHandleDatabase, *tmrHandle);
+error_exit:
 	return (error);
 }
 
@@ -169,24 +146,25 @@ saTmrSelectionObjectGet (
 {
 	struct tmrInstance *tmrInstance;
 	SaAisErrorT error = SA_AIS_OK;
-
-	/* DEBUG */
-	printf ("[DEBUG]: saTmrSelectionObjectGet\n");
+	int fd;
 
 	if (selectionObject == NULL) {
-		return (SA_AIS_ERR_INVALID_PARAM);
+		error = SA_AIS_ERR_INVALID_PARAM;
+		goto error_exit;
 	}
 
-	error = saHandleInstanceGet (&tmrHandleDatabase, tmrHandle, (void *)&tmrInstance);
+	error = hdb_error_to_sa(hdb_handle_get (&tmrHandleDatabase, tmrHandle, (void *)&tmrInstance));
 	if (error != SA_AIS_OK) {
-		return (error);
+		goto error_exit;
 	}
 
-	*selectionObject = coroipcc_fd_get (&tmrInstance->ipc_ctx);
+	error = coroipcc_fd_get (tmrInstance->ipc_handle, &fd);
+	*selectionObject = fd;
 
-	saHandleInstancePut (&tmrHandleDatabase, tmrHandle);
+	hdb_handle_put (&tmrHandleDatabase, tmrHandle);
 
-	return (SA_AIS_OK);
+error_exit:
+	return (error);
 }
 
 SaAisErrorT
@@ -197,8 +175,7 @@ saTmrDispatch (
 	SaTmrCallbacksT callbacks;
 	SaAisErrorT error = SA_AIS_OK;
 	struct tmrInstance *tmrInstance;
-	struct message_overlay dispatch_data;
-	int dispatch_avail;
+	coroipc_response_header_t *dispatch_data;
 	int timeout = 1;
 	int cont = 1;
 
@@ -208,11 +185,12 @@ saTmrDispatch (
 	    dispatchFlags != SA_DISPATCH_ALL &&
 	    dispatchFlags != SA_DISPATCH_BLOCKING)
 	{
-		return (SA_AIS_ERR_INVALID_PARAM);
+		error = SA_AIS_ERR_INVALID_PARAM;
+		goto error_exit;
 	}
 
-	error = saHandleInstanceGet (&tmrHandleDatabase, tmrHandle,
-		(void *)&tmrInstance);
+	error = hdb_error_to_sa(hdb_handle_get (&tmrHandleDatabase, tmrHandle,
+		(void *)&tmrInstance));
 	if (error != SA_AIS_OK) {
 		goto error_exit;
 	}
@@ -222,45 +200,37 @@ saTmrDispatch (
 	}
 
 	do {
-		dispatch_avail = coroipcc_dispatch_recv (tmrInstance->ipc_ctx,
-			(void *)&dispatch_data, timeout);
-
-		pthread_mutex_lock (&tmrInstance->dispatch_mutex);
-
-		if (tmrInstance->finalize == 1) {
-			error = SA_AIS_OK;
-			goto error_unlock;
+		error = coroipcc_dispatch_get (
+			tmrInstance->ipc_handle,
+			(void **)&dispatch_data,
+			timeout);
+		if (error == CS_ERR_BAD_HANDLE) {
+			error = CS_OK;
+			goto error_put;
+		}
+		if (error != CS_OK) {
+			goto error_put;
 		}
 
-		if (dispatch_avail == 0 && dispatchFlags == SA_DISPATCH_ALL) {
-			pthread_mutex_unlock (&tmrInstance->dispatch_mutex);
-			break;
+		if (dispatch_data == NULL) {
+			if (dispatchFlags == CPG_DISPATCH_ALL) {
+				break;
+			} else {
+				continue;
+			}
 		}
-		else if (dispatch_avail == 0) {
-			pthread_mutex_unlock (&tmrInstance->dispatch_mutex);
-			continue;
-		}
-
-		memset (&dispatch_data, 0, sizeof (struct message_overlay));
 
 		memcpy (&callbacks, &tmrInstance->callbacks,
 			sizeof (tmrInstance->callbacks));
 
-		pthread_mutex_unlock (&tmrInstance->dispatch_mutex);
-
-		/* DEBUG */
-		printf ("[DEBUG]: saTmrDispatch { id = %d }\n",
-			dispatch_data.header.id);
-
-		switch (dispatch_data.header.id)
-		{
+		switch (dispatch_data->id) {
 		case MESSAGE_RES_TMR_TIMEREXPIREDCALLBACK:
 			if (callbacks.saTmrTimerExpiredCallback == NULL) {
 				continue;
 			}
 
 			res_lib_tmr_timerexpiredcallback =
-				(struct res_lib_tmr_timerexpiredcallback *) &dispatch_data;
+				(struct res_lib_tmr_timerexpiredcallback *) dispatch_data;
 
 			callbacks.saTmrTimerExpiredCallback (
 				res_lib_tmr_timerexpiredcallback->timer_id,
@@ -271,6 +241,7 @@ saTmrDispatch (
 		default:
 			break;
 		}
+		coroipcc_dispatch_put (tmrInstance->ipc_handle);
 
 		switch (dispatchFlags)
 		{
@@ -284,10 +255,8 @@ saTmrDispatch (
 		}
 	} while (cont);
 
-error_unlock:
-	pthread_mutex_unlock (&tmrInstance->dispatch_mutex);
-
-	saHandleInstancePut (&tmrHandleDatabase, tmrHandle);
+error_put:
+	hdb_handle_put (&tmrHandleDatabase, tmrHandle);
 error_exit:
 	return (error);
 }
@@ -299,32 +268,24 @@ saTmrFinalize (
 	struct tmrInstance *tmrInstance;
 	SaAisErrorT error = SA_AIS_OK;
 
-	/* DEBUG */
-	printf ("[DEBUG]: saTmrFinalize\n");
-
-	error = saHandleInstanceGet (&tmrHandleDatabase, tmrHandle, (void *)&tmrInstance);
+	error = hdb_error_to_sa(hdb_handle_get (&tmrHandleDatabase, tmrHandle, (void *)&tmrInstance));
 	if (error != SA_AIS_OK) {
-		return (error);
+		goto error_exit;
 	}
 
-	pthread_mutex_lock (&tmrInstance->response_mutex);
-
 	if (tmrInstance->finalize) {
-		pthread_mutex_unlock (&tmrInstance->response_mutex);
-		saHandleInstancePut (&tmrHandleDatabase, tmrHandle);
-		return (SA_AIS_ERR_BAD_HANDLE);
+		hdb_handle_put (&tmrHandleDatabase, tmrHandle);
+		error = SA_AIS_ERR_BAD_HANDLE;
+		goto error_exit;
 	}
 
 	tmrInstance->finalize = 1;
 
-	coroipcc_service_disconnect (tmrInstance->ipc_ctx);
+	coroipcc_service_disconnect (tmrInstance->ipc_handle);
 
-	pthread_mutex_unlock (&tmrInstance->response_mutex);
-
-	/* tmrInstanceFinalize (tmrInstance); */
-
-	saHandleInstancePut (&tmrHandleDatabase, tmrHandle);
+	hdb_handle_put (&tmrHandleDatabase, tmrHandle);
 
+error_exit:
 	return (SA_AIS_OK);
 }
 
@@ -343,22 +304,35 @@ saTmrTimerStart (
 	struct res_lib_tmr_timerstart res_lib_tmr_timerstart;
 	struct iovec iov;
 
-	/* DEBUG */
-	printf ("[DEBUG]: saTmrTimerStart { data=%p }\n",
-		(void *)(timerData));
-
-	if (timerAttributes == NULL) {
-		return (SA_AIS_ERR_INVALID_PARAM);
+	if ((timerAttributes == NULL) || (callTime == NULL) || (timerId == NULL)) {
+		error = SA_AIS_ERR_INVALID_PARAM;
+		goto error_exit;
 	}
 
 	if ((timerAttributes->type != SA_TIME_ABSOLUTE) &&
 	    (timerAttributes->type != SA_TIME_DURATION)) {
-		return (SA_AIS_ERR_INVALID_PARAM);
+		error = SA_AIS_ERR_INVALID_PARAM;
+		goto error_exit;
 	}
 
-	error = saHandleInstanceGet (&tmrHandleDatabase, tmrHandle, (void *)&tmrInstance);
+	if (timerAttributes->initialExpirationTime < 0) {
+		error = SA_AIS_ERR_INVALID_PARAM;
+		goto error_exit;
+	}
+
+	if (timerAttributes->timerPeriodDuration < 0) {
+		error = SA_AIS_ERR_INVALID_PARAM;
+		goto error_exit;
+	}
+
+	error = hdb_error_to_sa(hdb_handle_get (&tmrHandleDatabase, tmrHandle, (void *)&tmrInstance));
 	if (error != SA_AIS_OK) {
-		return (error);
+		goto error_exit;
+	}
+
+	if (tmrInstance->callbacks.saTmrTimerExpiredCallback == NULL) {
+		error = SA_AIS_ERR_INIT;
+		goto error_put;
 	}
 
 	req_lib_tmr_timerstart.header.size =
@@ -371,19 +345,15 @@ saTmrTimerStart (
 
 	req_lib_tmr_timerstart.timer_data = (void *)timerData;
 
-	iov.iov_base = &req_lib_tmr_timerstart;
+	iov.iov_base = (void *)&req_lib_tmr_timerstart;
 	iov.iov_len = sizeof (struct req_lib_tmr_timerstart);
 
-	pthread_mutex_lock (&tmrInstance->response_mutex);
-
-	error = coroipcc_msg_send_reply_receive (tmrInstance->ipc_ctx,
+	error = coroipcc_msg_send_reply_receive (tmrInstance->ipc_handle,
 		&iov,
 		1,
 		&res_lib_tmr_timerstart,
 		sizeof (struct res_lib_tmr_timerstart));
 
-	pthread_mutex_unlock (&tmrInstance->response_mutex);
-
 	*timerId = (SaTmrTimerIdT)(res_lib_tmr_timerstart.timer_id);
 	*callTime = (SaTimeT)(res_lib_tmr_timerstart.call_time);
 
@@ -391,7 +361,9 @@ saTmrTimerStart (
 		error = res_lib_tmr_timerstart.header.error;
 	}
 
-	saHandleInstancePut (&tmrHandleDatabase, tmrHandle);
+error_put:
+	hdb_handle_put (&tmrHandleDatabase, tmrHandle);
+error_exit:
 	return (error);
 }
 
@@ -409,17 +381,21 @@ saTmrTimerReschedule (
 	struct res_lib_tmr_timerreschedule res_lib_tmr_timerreschedule;
 	struct iovec iov;
 
-	/* DEBUG */
-	printf ("[DEBUG]: saTmrTimerReschedule { id=%u }\n",
-		(unsigned int)(timerId));
-
-	if (timerAttributes == NULL) {
-		return (SA_AIS_ERR_INVALID_PARAM);
+	if ((timerAttributes == NULL) || (callTime == NULL)) {
+		error = SA_AIS_ERR_INVALID_PARAM;
+		goto error_exit;
 	}
 
-	error = saHandleInstanceGet (&tmrHandleDatabase, tmrHandle, (void *)&tmrInstance);
+	if ((timerAttributes->type != SA_TIME_ABSOLUTE) &&
+	    (timerAttributes->type != SA_TIME_DURATION))
+	{
+		error = SA_AIS_ERR_INVALID_PARAM;
+		goto error_exit;
+	}
+
+	error = hdb_error_to_sa(hdb_handle_get (&tmrHandleDatabase, tmrHandle, (void *)&tmrInstance));
 	if (error != SA_AIS_OK) {
-		return (error);
+		goto error_exit;
 	}
 
 	req_lib_tmr_timerreschedule.header.size =
@@ -432,25 +408,22 @@ saTmrTimerReschedule (
 	memcpy (&req_lib_tmr_timerreschedule.timer_attributes,
 		timerAttributes, sizeof (SaTmrTimerAttributesT));
 
-	iov.iov_base = &req_lib_tmr_timerreschedule,
+	iov.iov_base = (void *)&req_lib_tmr_timerreschedule,
 	iov.iov_len = sizeof (struct req_lib_tmr_timerreschedule);
 
-	pthread_mutex_lock (&tmrInstance->response_mutex);
-
-	error = coroipcc_msg_send_reply_receive (tmrInstance->ipc_ctx,
+	error = coroipcc_msg_send_reply_receive (tmrInstance->ipc_handle,
 		&iov,
 		1,
 		&res_lib_tmr_timerreschedule,
 		sizeof (struct res_lib_tmr_timerreschedule));
 
-	pthread_mutex_unlock (&tmrInstance->response_mutex);
-
 	if (res_lib_tmr_timerreschedule.header.error != SA_AIS_OK) {
 		error = res_lib_tmr_timerreschedule.header.error;
 	}
 
-	saHandleInstancePut (&tmrHandleDatabase, tmrHandle);
+	hdb_handle_put (&tmrHandleDatabase, tmrHandle);
 
+error_exit:
 	return (error);
 }
 
@@ -467,11 +440,12 @@ saTmrTimerCancel (
 	struct res_lib_tmr_timercancel res_lib_tmr_timercancel;
 	struct iovec iov;
 
-	/* DEBUG */
-	printf ("[DEBUG]: saTmrTimerCancel { id=%u }\n",
-		(unsigned int)(timerId));
+	if (timerDataP == NULL) {
+		error = SA_AIS_ERR_INVALID_PARAM;
+		goto error_exit;
+	}
 
-	error = saHandleInstanceGet (&tmrHandleDatabase, tmrHandle, (void *)&tmrInstance);
+	error = hdb_error_to_sa(hdb_handle_get (&tmrHandleDatabase, tmrHandle, (void *)&tmrInstance));
 	if (error != SA_AIS_OK) {
 		goto error_exit;
 	}
@@ -483,19 +457,15 @@ saTmrTimerCancel (
 
 	req_lib_tmr_timercancel.timer_id = timerId;
 
-	iov.iov_base = &req_lib_tmr_timercancel,
+	iov.iov_base = (void *)&req_lib_tmr_timercancel,
 	iov.iov_len = sizeof (struct req_lib_tmr_timercancel);
 
-	pthread_mutex_lock (&tmrInstance->response_mutex);
-
-	error = coroipcc_msg_send_reply_receive (tmrInstance->ipc_ctx,
+	error = coroipcc_msg_send_reply_receive (tmrInstance->ipc_handle,
 		&iov,
 		1,
 		&res_lib_tmr_timercancel,
 		sizeof (struct res_lib_tmr_timercancel));
 
-	pthread_mutex_unlock (&tmrInstance->response_mutex);
-
 	if (res_lib_tmr_timercancel.header.error != SA_AIS_OK) {
 		error = res_lib_tmr_timercancel.header.error;
 	}
@@ -520,13 +490,9 @@ saTmrPeriodicTimerSkip (
 	struct res_lib_tmr_periodictimerskip res_lib_tmr_periodictimerskip;
 	struct iovec iov;
 
-	/* DEBUG */
-	printf ("[DEBUG]: saTmrPeriodicTimerSkip { id=%u }\n",
-		(unsigned int)(timerId));
-
-	error = saHandleInstanceGet (&tmrHandleDatabase, tmrHandle, (void *)&tmrInstance);
+	error = hdb_error_to_sa(hdb_handle_get (&tmrHandleDatabase, tmrHandle, (void *)&tmrInstance));
 	if (error != SA_AIS_OK) {
-		return (error);
+		goto error_exit;
 	}
 
 	req_lib_tmr_periodictimerskip.header.size =
@@ -536,23 +502,20 @@ saTmrPeriodicTimerSkip (
 
 	req_lib_tmr_periodictimerskip.timer_id = timerId;
 
-	iov.iov_base = &req_lib_tmr_periodictimerskip,
+	iov.iov_base = (void *)&req_lib_tmr_periodictimerskip,
 	iov.iov_len = sizeof (struct req_lib_tmr_periodictimerskip);
 
-	pthread_mutex_lock (&tmrInstance->response_mutex);
-
-	error = coroipcc_msg_send_reply_receive (tmrInstance->ipc_ctx,
+	error = coroipcc_msg_send_reply_receive (tmrInstance->ipc_handle,
 		&iov,
 		1,
 		&res_lib_tmr_periodictimerskip,
 		sizeof (struct res_lib_tmr_periodictimerskip));
 
-	pthread_mutex_unlock (&tmrInstance->response_mutex);
-
 	if (res_lib_tmr_periodictimerskip.header.error != SA_AIS_OK) {
 		error = res_lib_tmr_periodictimerskip.header.error;
 	}
 
+error_exit:
 	return (error);
 }
 
@@ -569,18 +532,14 @@ saTmrTimerRemainingTimeGet (
 	struct res_lib_tmr_timerremainingtimeget res_lib_tmr_timerremainingtimeget;
 	struct iovec iov;
 
-	/* DEBUG */
-	printf ("[DEBUG]: saTimerRemainingTimeGet { id=%u }\n",
-		(unsigned int)(timerId));
-
 	if (remainingTime == NULL) {
 		error = SA_AIS_ERR_INVALID_PARAM;
 		goto error_exit;
 	}
 
-	error = saHandleInstanceGet (&tmrHandleDatabase, tmrHandle, (void *)&tmrInstance);
+	error = hdb_error_to_sa(hdb_handle_get (&tmrHandleDatabase, tmrHandle, (void *)&tmrInstance));
 	if (error != SA_AIS_OK) {
-		return (error);
+		goto error_exit;
 	}
 
 	req_lib_tmr_timerremainingtimeget.header.size =
@@ -590,19 +549,15 @@ saTmrTimerRemainingTimeGet (
 
 	req_lib_tmr_timerremainingtimeget.timer_id = timerId;
 
-	iov.iov_base = &req_lib_tmr_timerremainingtimeget,
+	iov.iov_base = (void *)&req_lib_tmr_timerremainingtimeget,
 	iov.iov_len = sizeof (struct req_lib_tmr_timerremainingtimeget);
 
-	pthread_mutex_lock (&tmrInstance->response_mutex);
-
-	error = coroipcc_msg_send_reply_receive (tmrInstance->ipc_ctx,
+	error = coroipcc_msg_send_reply_receive (tmrInstance->ipc_handle,
 		&iov,
 		1,
 		&res_lib_tmr_timerremainingtimeget,
 		sizeof (struct res_lib_tmr_timerremainingtimeget));
 
-	pthread_mutex_unlock (&tmrInstance->response_mutex);
-
 	if (res_lib_tmr_timerremainingtimeget.header.error != SA_AIS_OK) {
 		error = res_lib_tmr_timerremainingtimeget.header.error;
 	}
@@ -626,17 +581,12 @@ saTmrTimerAttributesGet (
 	struct res_lib_tmr_timerattributesget res_lib_tmr_timerattributesget;
 	struct iovec iov;
 
-
-	/* DEBUG */
-	printf ("[DEBUG]: saTmrTimerAttributesGet { id=%u }\n",
-		(unsigned int)(timerId));
-
 	if (timerAttributes == NULL) {
 		error = SA_AIS_ERR_INVALID_PARAM;
 		goto error_exit;
 	}
 
-	error = saHandleInstanceGet (&tmrHandleDatabase, tmrHandle, (void *)&tmrInstance);
+	error = hdb_error_to_sa(hdb_handle_get (&tmrHandleDatabase, tmrHandle, (void *)&tmrInstance));
 	if (error != SA_AIS_OK) {
 		goto error_exit;
 	}
@@ -648,19 +598,15 @@ saTmrTimerAttributesGet (
 
 	req_lib_tmr_timerattributesget.timer_id = timerId;
 
-	iov.iov_base = &req_lib_tmr_timerattributesget,
+	iov.iov_base = (void *)&req_lib_tmr_timerattributesget,
 	iov.iov_len = sizeof (struct req_lib_tmr_timerattributesget);
 
-	pthread_mutex_lock (&tmrInstance->response_mutex);
-
-	error = coroipcc_msg_send_reply_receive (tmrInstance->ipc_ctx,
+	error = coroipcc_msg_send_reply_receive (tmrInstance->ipc_handle,
 		&iov,
 		1,
 		&res_lib_tmr_timerattributesget,
 		sizeof (struct res_lib_tmr_timerattributesget));
 
-	pthread_mutex_unlock (&tmrInstance->response_mutex);
-
 	if (res_lib_tmr_timerattributesget.header.error != SA_AIS_OK) {
 		error = res_lib_tmr_timerattributesget.header.error;
 	}
@@ -684,15 +630,12 @@ saTmrTimeGet (
 	struct res_lib_tmr_timeget res_lib_tmr_timeget;
 	struct iovec iov;
 
-	/* DEBUG */
-	printf ("[DEBUG]: saTmrTimeGet\n");
-
 	if (currentTime == NULL) {
 		error = SA_AIS_ERR_INVALID_PARAM;
 		goto error_exit;
 	}
 
-	error = saHandleInstanceGet (&tmrHandleDatabase, tmrHandle, (void *)&tmrInstance);
+	error = hdb_error_to_sa(hdb_handle_get (&tmrHandleDatabase, tmrHandle, (void *)&tmrInstance));
 	if (error != SA_AIS_OK) {
 		goto error_exit;
 	}
@@ -702,19 +645,15 @@ saTmrTimeGet (
 	req_lib_tmr_timeget.header.id =
 		MESSAGE_REQ_TMR_TIMEGET;
 
-	iov.iov_base = &req_lib_tmr_timeget,
+	iov.iov_base = (void *)&req_lib_tmr_timeget,
 	iov.iov_len = sizeof (struct req_lib_tmr_timeget);
 
-	pthread_mutex_lock (&tmrInstance->response_mutex);
-
-	error = coroipcc_msg_send_reply_receive (tmrInstance->ipc_ctx,
+	error = coroipcc_msg_send_reply_receive (tmrInstance->ipc_handle,
 		&iov,
 		1,
 		&res_lib_tmr_timeget,
 		sizeof (struct res_lib_tmr_timeget));
 
-	pthread_mutex_unlock (&tmrInstance->response_mutex);
-
 	if (res_lib_tmr_timeget.header.error != SA_AIS_OK) {
 		error = res_lib_tmr_timeget.header.error;
 	}
@@ -738,15 +677,12 @@ saTmrClockTickGet (
 	struct res_lib_tmr_clocktickget res_lib_tmr_clocktickget;
 	struct iovec iov;
 
-	/* DEBUG */
-	printf ("[DEBUG]: saTmrClockTickGet\n");
-
 	if (clockTick == NULL) {
 		error = SA_AIS_ERR_INVALID_PARAM;
 		goto error_exit;
 	}
 
-	error = saHandleInstanceGet (&tmrHandleDatabase, tmrHandle, (void *)&tmrInstance);
+	error = hdb_error_to_sa(hdb_handle_get (&tmrHandleDatabase, tmrHandle, (void *)&tmrInstance));
 	if (error != SA_AIS_OK) {
 		goto error_exit;
 	}
@@ -756,19 +692,15 @@ saTmrClockTickGet (
 	req_lib_tmr_clocktickget.header.id =
 		MESSAGE_REQ_TMR_CLOCKTICKGET;
 
-	iov.iov_base = &req_lib_tmr_clocktickget,
+	iov.iov_base = (void *)&req_lib_tmr_clocktickget,
 	iov.iov_len = sizeof (struct req_lib_tmr_clocktickget);
 
-	pthread_mutex_lock (&tmrInstance->response_mutex);
-
-	error = coroipcc_msg_send_reply_receive (tmrInstance->ipc_ctx,
+	error = coroipcc_msg_send_reply_receive (tmrInstance->ipc_handle,
 		&iov,
 		1,
 		&res_lib_tmr_clocktickget,
 		sizeof (struct res_lib_tmr_clocktickget));
 
-	pthread_mutex_unlock (&tmrInstance->response_mutex);
-
 	if (res_lib_tmr_clocktickget.header.error != SA_AIS_OK) {
 		error = res_lib_tmr_clocktickget.header.error;
 	}
