@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 2002-2006 MontaVista Software, Inc.
- * Copyright (c) 2006-2007 Red Hat, Inc.
+ * Copyright (c) 2006-2007, 2009 Red Hat, Inc.
  * Copyright (C) 2006 Sun Microsystems, Inc.
  *
  * All rights reserved.
@@ -8,7 +8,7 @@
  * Author: Steven Dake (sdake@redhat.com)
  *
  * This software licensed under BSD license, the text of which follows:
- * 
+ *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions are met:
  *
@@ -33,6 +33,8 @@
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
  * THE POSSIBILITY OF SUCH DAMAGE.
  */
+#include <config.h>
+
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <sys/un.h>
@@ -51,13 +53,15 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <errno.h>
-#include <signal.h>
 #include <time.h>
 #include <unistd.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
 
-#include <corosync/ipc_gen.h>
+#include <corosync/corotypes.h>
+#include <corosync/coroipc_types.h>
+#include <corosync/coroipcc.h>
+#include <corosync/corodefs.h>
 #include <corosync/mar_gen.h>
 #include <corosync/swab.h>
 #include <corosync/list.h>
@@ -71,7 +75,7 @@
 #include "../include/mar_clm.h"
 #include "clm.h"
 
-LOGSYS_DECLARE_SUBSYS ("CLM", LOG_INFO);
+LOGSYS_DECLARE_SUBSYS ("CLM");
 
 
 enum clm_message_req_types {
@@ -122,12 +126,15 @@ static SaClmClusterNodeT *clm_services_api_nodeid_saf_get (unsigned int node_id)
  */
 static void clm_confchg_fn (
 	enum totem_configuration_type configuration_type,
-	unsigned int *member_list, int member_list_entries,
-	unsigned int *left_list, int left_list_entries,
-	unsigned int *joined_list, int joined_list_entries,
-	struct memb_ring_id *ring_id);
+	const unsigned int *member_list, size_t member_list_entries,
+	const unsigned int *left_list, size_t left_list_entries,
+	const unsigned int *joined_list, size_t joined_list_entries,
+	const struct memb_ring_id *ring_id);
 
-static void clm_sync_init (void);
+static void clm_sync_init (
+	const unsigned int *member_list,
+	size_t member_list_entries,
+	const struct memb_ring_id *ring_id);
 
 static int clm_sync_process (void);
 
@@ -144,18 +151,18 @@ static int clm_lib_init_fn (void *conn);
 static int clm_lib_exit_fn (void *conn);
 
 static void message_handler_req_exec_clm_nodejoin (
-	void *message,
+	const void *message,
 	unsigned int nodeid);
 
 static void exec_clm_nodejoin_endian_convert (void *msg);
 
-static void message_handler_req_lib_clm_clustertrack (void *conn, void *message);
+static void message_handler_req_lib_clm_clustertrack (void *conn, const void *message);
 
-static void message_handler_req_lib_clm_trackstop (void *conn, void *message);
+static void message_handler_req_lib_clm_trackstop (void *conn, const void *message);
 
-static void message_handler_req_lib_clm_nodeget (void *conn, void *message);
+static void message_handler_req_lib_clm_nodeget (void *conn, const void *message);
 
-static void message_handler_req_lib_clm_nodegetasync (void *conn, void *message);
+static void message_handler_req_lib_clm_nodegetasync (void *conn, const void *message);
 
 struct clm_pd {
 	unsigned char track_flags;
@@ -171,26 +178,18 @@ static struct corosync_lib_handler clm_lib_engine[] =
 {
 	{ /* 0 */
 		.lib_handler_fn				= message_handler_req_lib_clm_clustertrack,
-		.response_size				= sizeof (struct res_lib_clm_clustertrack),
-		.response_id				= MESSAGE_RES_CLM_TRACKSTART, // TODO RESPONSE
 		.flow_control				= COROSYNC_LIB_FLOW_CONTROL_NOT_REQUIRED
 	},
 	{ /* 1 */
 		.lib_handler_fn				= message_handler_req_lib_clm_trackstop,
-		.response_size				= sizeof (struct res_lib_clm_trackstop),
-		.response_id				= MESSAGE_RES_CLM_TRACKSTOP, // TODO RESPONSE
 		.flow_control				= COROSYNC_LIB_FLOW_CONTROL_NOT_REQUIRED
 	},
 	{ /* 2 */
 		.lib_handler_fn				= message_handler_req_lib_clm_nodeget,
-		.response_size				= sizeof (struct res_clm_nodeget),
-		.response_id				= MESSAGE_RES_CLM_NODEGET, // TODO RESPONSE
 		.flow_control				= COROSYNC_LIB_FLOW_CONTROL_NOT_REQUIRED
 	},
 	{ /* 3 */
 		.lib_handler_fn				= message_handler_req_lib_clm_nodegetasync,
-		.response_size				= sizeof (struct res_clm_nodegetasync),
-		.response_id				= MESSAGE_RES_CLM_NODEGETCALLBACK, // TODO RESPONSE
 		.flow_control				= COROSYNC_LIB_FLOW_CONTROL_NOT_REQUIRED
 	}
 };
@@ -202,12 +201,12 @@ static struct corosync_exec_handler clm_exec_engine[] =
 		.exec_endian_convert_fn	= exec_clm_nodejoin_endian_convert
 	}
 };
-	
+
 struct corosync_service_engine clm_service_engine = {
 	.name			= "openais cluster membership service B.01.01",
 	.id			= CLM_SERVICE,
 	.private_data_size	= sizeof (struct clm_pd),
-	.flow_control		= COROSYNC_LIB_FLOW_CONTROL_NOT_REQUIRED, 
+	.flow_control		= COROSYNC_LIB_FLOW_CONTROL_NOT_REQUIRED,
 	.lib_init_fn		= clm_lib_init_fn,
 	.lib_exit_fn		= clm_lib_exit_fn,
 	.lib_engine		= clm_lib_engine,
@@ -218,6 +217,7 @@ struct corosync_service_engine clm_service_engine = {
 	.exec_engine		= clm_exec_engine,
 	.exec_engine_count	= sizeof (clm_exec_engine) / sizeof (struct corosync_exec_handler),
 	.confchg_fn		= clm_confchg_fn,
+	.sync_mode		= CS_SYNC_V1,
 	.sync_init		= clm_sync_init,
 	.sync_process		= clm_sync_process,
 	.sync_activate		= clm_sync_activate,
@@ -272,7 +272,15 @@ static struct corosync_service_engine *clm_get_service_engine_ver0 (void)
 	return (&clm_service_engine);
 }
 
-__attribute__ ((constructor)) static void clm_comp_register (void) {
+
+#ifdef OPENAIS_SOLARIS
+void corosync_lcr_component_register (void);
+
+void corosync_lcr_component_register (void) {
+#else
+__attribute__ ((constructor)) static void corosync_lcr_component_register (void)
+{
+#endif
 	lcr_interfaces_set (&openais_clm_ver0[0], &clm_service_engine_iface);
 	lcr_interfaces_set (&openais_clm_ver0[1], &clm_services_api_ver1);
 
@@ -280,7 +288,7 @@ __attribute__ ((constructor)) static void clm_comp_register (void) {
 }
 
 struct req_exec_clm_nodejoin {
-	mar_req_header_t header __attribute__((aligned(8)));
+	coroipc_request_header_t header __attribute__((aligned(8)));
 	mar_clm_cluster_node_t cluster_node __attribute__((aligned(8)));
 };
 
@@ -291,7 +299,6 @@ static void my_cluster_node_load (void)
 	char **status;
 	const char *iface_string;
 
-printf ("nodeid get %d\n", api->totem_nodeid_get());
 	api->totem_ifaces_get (
 		api->totem_nodeid_get (),
 		interfaces,
@@ -326,6 +333,10 @@ printf ("nodeid get %d\n", api->totem_nodeid_get());
 
 static int clm_exec_init_fn (struct corosync_api_v1 *corosync_api)
 {
+#ifdef OPENAIS_SOLARIS
+	logsys_subsys_init();
+#endif
+
 	api = corosync_api;
 
 	memset (cluster_node_entries, 0,
@@ -346,7 +357,7 @@ static int clm_exec_init_fn (struct corosync_api_v1 *corosync_api)
 		int mib[2] = { CTL_KERN, KERN_BOOTTIME };
 		struct timeval boot_time;
 		size_t size = sizeof(boot_time);
-		
+
 		if ( sysctl(mib, 2, &boot_time, &size, NULL, 0) == -1 )
 			boot_time.tv_sec = time (NULL);
 		 /* (currenttime (s) - uptime (s)) * 1 billion (ns) / 1 (s) */
@@ -368,7 +379,6 @@ static int clm_exec_init_fn (struct corosync_api_v1 *corosync_api)
 
 	cluster_node_count = 1;
 
-//TODO	main_clm_get_by_nodeid = clm_get_by_nodeid;
 	return (0);
 }
 
@@ -448,14 +458,14 @@ static void library_notification_send (
 		/*
 		 * Send notifications to all CLM listeners
 		 */
-		api->ipc_response_send (
+		api->ipc_dispatch_send (
 			clm_pd->conn,
 			&res_lib_clm_clustertrack,
 			sizeof (struct res_lib_clm_clustertrack));
 	}
 }
 
-static void notification_join (mar_clm_cluster_node_t *cluster_node)
+static void notification_join (const mar_clm_cluster_node_t *cluster_node)
 {
 	mar_clm_cluster_notification_t notification;
 
@@ -465,7 +475,7 @@ static void notification_join (mar_clm_cluster_node_t *cluster_node)
 	notification.cluster_change = SA_CLM_NODE_JOINED;
 	notification.cluster_node.member = 1;
 	memcpy (&notification.cluster_node, cluster_node,
-		sizeof (mar_clm_cluster_node_t)); 
+		sizeof (mar_clm_cluster_node_t));
 	library_notification_send (&notification, 1);
 }
 
@@ -481,7 +491,7 @@ static void lib_notification_leave (unsigned int *nodes, int nodes_entries)
 	for (notify_count = 0, i = 0; i < cluster_node_count; i++) {
 		for (j = 0; j < nodes_entries; j++) {
 			if (cluster_node_entries[i].node_id == nodes[j]) {
-				memcpy (&cluster_notification[notify_count].cluster_node, 
+				memcpy (&cluster_notification[notify_count].cluster_node,
 					&cluster_node_entries[i],
 					sizeof (mar_clm_cluster_node_t));
 				cluster_notification[notify_count].cluster_change = SA_CLM_NODE_LEFT;
@@ -520,14 +530,14 @@ static int clm_nodejoin_send (void)
 	int result;
 
 	req_exec_clm_nodejoin.header.size = sizeof (struct req_exec_clm_nodejoin);
-	req_exec_clm_nodejoin.header.id = 
+	req_exec_clm_nodejoin.header.id =
 		SERVICE_ID_MAKE (CLM_SERVICE, MESSAGE_REQ_EXEC_CLM_NODEJOIN);
 
 	my_cluster_node.initial_view_number = view_initial;
 
 	memcpy (&req_exec_clm_nodejoin.cluster_node, &my_cluster_node,
 		sizeof (mar_clm_cluster_node_t));
-	
+
 	req_exec_clm_iovec.iov_base = (char *)&req_exec_clm_nodejoin;
 	req_exec_clm_iovec.iov_len = sizeof (req_exec_clm_nodejoin);
 
@@ -538,10 +548,10 @@ static int clm_nodejoin_send (void)
 
 static void clm_confchg_fn (
 	enum totem_configuration_type configuration_type,
-	unsigned int *member_list, int member_list_entries,
-	unsigned int *left_list, int left_list_entries,
-	unsigned int *joined_list, int joined_list_entries,
-	struct memb_ring_id *ring_id)
+	const unsigned int *member_list, size_t member_list_entries,
+	const unsigned int *left_list, size_t left_list_entries,
+	const unsigned int *joined_list, size_t joined_list_entries,
+	const struct memb_ring_id *ring_id)
 {
 	int i;
 	unsigned int node_ids[PROCESSOR_COUNT_MAX];
@@ -551,19 +561,19 @@ static void clm_confchg_fn (
 		view_initial = ring_id->seq / 4;
 	}
 
-	log_printf (LOG_LEVEL_NOTICE, "CLM CONFIGURATION CHANGE\n");
-	log_printf (LOG_LEVEL_NOTICE, "New Configuration:\n");
+	log_printf (LOGSYS_LEVEL_NOTICE, "CLM CONFIGURATION CHANGE\n");
+	log_printf (LOGSYS_LEVEL_NOTICE, "New Configuration:\n");
 	for (i = 0; i < member_list_entries; i++) {
-		log_printf (LOG_LEVEL_NOTICE, "\t%s\n", api->totem_ifaces_print (member_list[i]));
+		log_printf (LOGSYS_LEVEL_NOTICE, "\t%s\n", api->totem_ifaces_print (member_list[i]));
 	}
-	log_printf (LOG_LEVEL_NOTICE, "Members Left:\n");
+	log_printf (LOGSYS_LEVEL_NOTICE, "Members Left:\n");
 	for (i = 0; i < left_list_entries; i++) {
-		log_printf (LOG_LEVEL_NOTICE, "\t%s\n", api->totem_ifaces_print (left_list[i]));
+		log_printf (LOGSYS_LEVEL_NOTICE, "\t%s\n", api->totem_ifaces_print (left_list[i]));
 	}
 
-	log_printf (LOG_LEVEL_NOTICE, "Members Joined:\n");
+	log_printf (LOGSYS_LEVEL_NOTICE, "Members Joined:\n");
 	for (i = 0; i < joined_list_entries; i++) {
-		log_printf (LOG_LEVEL_NOTICE, "\t%s\n", api->totem_ifaces_print (joined_list[i]));
+		log_printf (LOGSYS_LEVEL_NOTICE, "\t%s\n", api->totem_ifaces_print (joined_list[i]));
 	}
 
 	for (i = 0; i < left_list_entries; i++) {
@@ -582,7 +592,10 @@ static void clm_confchg_fn (
 /*
  * This is a noop for this service
  */
-static void clm_sync_init (void)
+static void clm_sync_init (
+	const unsigned int *member_list,
+	size_t member_list_entries,
+	const struct memb_ring_id *ring_id)
 {
 	return;
 }
@@ -620,21 +633,21 @@ static void exec_clm_nodejoin_endian_convert (void *msg)
 {
 	struct req_exec_clm_nodejoin *node_join = msg;
 
-	swab_mar_req_header_t (&node_join->header);
+	swab_coroipc_request_header_t (&node_join->header);
 	swab_mar_clm_cluster_node_t (&node_join->cluster_node);
 }
 
 static void message_handler_req_exec_clm_nodejoin (
-	void *message,
+	const void *message,
 	unsigned int nodeid)
 {
-	struct req_exec_clm_nodejoin *req_exec_clm_nodejoin = (struct req_exec_clm_nodejoin *)message;
+	const struct req_exec_clm_nodejoin *req_exec_clm_nodejoin = message;
 	int found = 0;
 	int i;
 
-	log_printf (LOG_LEVEL_NOTICE, "got nodejoin message %s\n",
+	log_printf (LOGSYS_LEVEL_DEBUG, "got nodejoin message %s\n",
 		req_exec_clm_nodejoin->cluster_node.node_name.value);
-	
+
 	/*
 	 * Determine if nodejoin already received
 	 */
@@ -659,16 +672,19 @@ static void message_handler_req_exec_clm_nodejoin (
 
 static int clm_lib_init_fn (void *conn)
 {
-	log_printf (LOG_LEVEL_DEBUG, "Got request to initalize cluster membership service.\n");
 	struct clm_pd *clm_pd = (struct clm_pd *)api->ipc_private_data_get (conn);
 
+	log_printf (LOGSYS_LEVEL_DEBUG, "Got request to initalize cluster membership service.\n");
+
 	list_init (&clm_pd->list);
 	clm_pd->conn = conn;
 
 	return (0);
 }
 
-static void message_handler_req_lib_clm_clustertrack (void *conn, void *msg)
+static void message_handler_req_lib_clm_clustertrack (
+	void *conn,
+	const void *msg)
 {
 	struct req_lib_clm_clustertrack *req_lib_clm_clustertrack = (struct req_lib_clm_clustertrack *)msg;
 	struct res_lib_clm_clustertrack res_lib_clm_clustertrack;
@@ -695,7 +711,7 @@ static void message_handler_req_lib_clm_clustertrack (void *conn, void *msg)
 		}
 		res_lib_clm_clustertrack.number_of_items = cluster_node_count;
 	}
-	
+
 	/*
 	 * Record requests for cluster tracking
 	 */
@@ -722,7 +738,9 @@ static void message_handler_req_lib_clm_clustertrack (void *conn, void *msg)
 }
 
 
-static void message_handler_req_lib_clm_trackstop (void *conn, void *msg)
+static void message_handler_req_lib_clm_trackstop (
+	void *conn,
+	const void *msg)
 {
 	struct res_lib_clm_trackstop res_lib_clm_trackstop;
 	struct clm_pd *clm_pd = (struct clm_pd *)api->ipc_private_data_get (conn);
@@ -744,7 +762,9 @@ static void message_handler_req_lib_clm_trackstop (void *conn, void *msg)
 		sizeof (struct res_lib_clm_trackstop));
 }
 
-static void message_handler_req_lib_clm_nodeget (void *conn, void *msg)
+static void message_handler_req_lib_clm_nodeget (
+	void *conn,
+	const void *msg)
 {
 	struct req_lib_clm_nodeget *req_lib_clm_nodeget = (struct req_lib_clm_nodeget *)msg;
 	struct res_clm_nodeget res_clm_nodeget;
@@ -752,16 +772,16 @@ static void message_handler_req_lib_clm_nodeget (void *conn, void *msg)
 	int valid = 0;
 	int i;
 
-	log_printf (LOG_LEVEL_NOTICE, "nodeget: trying to find node %x\n",
+	log_printf (LOGSYS_LEVEL_NOTICE, "nodeget: trying to find node %x\n",
 		(int)req_lib_clm_nodeget->node_id);
 
 	if (req_lib_clm_nodeget->node_id == SA_CLM_LOCAL_NODE_ID) {
 		cluster_node = &cluster_node_entries[0];
 		valid = 1;
-	} else 
+	} else
 	for (i = 0; i < cluster_node_count; i++) {
 		if (cluster_node_entries[i].node_id == req_lib_clm_nodeget->node_id) {
-			log_printf (LOG_LEVEL_DEBUG, "found host that matches one desired in nodeget.\n");
+			log_printf (LOGSYS_LEVEL_DEBUG, "found host that matches one desired in nodeget.\n");
 			cluster_node = &cluster_node_entries[i];
 			valid = 1;
 			break;
@@ -779,7 +799,9 @@ static void message_handler_req_lib_clm_nodeget (void *conn, void *msg)
 	api->ipc_response_send (conn, &res_clm_nodeget, sizeof (struct res_clm_nodeget));
 }
 
-static void message_handler_req_lib_clm_nodegetasync (void *conn, void *msg)
+static void message_handler_req_lib_clm_nodegetasync (
+	void *conn,
+	const void *msg)
 {
 	struct req_lib_clm_nodegetasync *req_lib_clm_nodegetasync = (struct req_lib_clm_nodegetasync *)msg;
 	struct res_clm_nodegetasync res_clm_nodegetasync;
@@ -788,16 +810,16 @@ static void message_handler_req_lib_clm_nodegetasync (void *conn, void *msg)
 	int error = SA_AIS_ERR_INVALID_PARAM;
 	int i;
 
-	log_printf (LOG_LEVEL_DEBUG, "nodeget: trying to find node %x\n",
+	log_printf (LOGSYS_LEVEL_DEBUG, "nodeget: trying to find node %x\n",
 		(int)req_lib_clm_nodegetasync->node_id);
 
 	if (req_lib_clm_nodegetasync->node_id == SA_CLM_LOCAL_NODE_ID) {
 		cluster_node = &cluster_node_entries[0];
 		error = SA_AIS_OK;
-	} else 
+	} else
 	for (i = 0; i < cluster_node_count; i++) {
 		if (cluster_node_entries[i].node_id == req_lib_clm_nodegetasync->node_id) {
-			log_printf (LOG_LEVEL_DEBUG, "found host that matches one desired in nodeget.\n");
+			log_printf (LOGSYS_LEVEL_DEBUG, "found host that matches one desired in nodeget.\n");
 			cluster_node = &cluster_node_entries[i];
 			error = SA_AIS_OK;
 			break;
